@@ -1,4 +1,4 @@
-/*	$OpenBSD: enqueue.c,v 1.112 2016/02/03 08:03:21 gilles Exp $	*/
+/*	$OpenBSD: enqueue.c,v 1.113 2016/07/03 14:30:33 gilles Exp $	*/
 
 /*
  * Copyright (c) 2005 Henning Brauer <henning@bulabula.org>
@@ -189,7 +189,7 @@ enqueue(int argc, char *argv[], FILE *ofp)
 	save_argv = argv;
 
 	while ((ch = getopt(argc, argv,
-	    "A:B:b:E::e:F:f:iJ::L:mN:o:p:qR:StvV:x")) != -1) {
+	    "A:B:b:E::e:F:f:iJ::L:mN:o:p:qr:R:StvV:x")) != -1) {
 		switch (ch) {
 		case 'f':
 			fake_from = optarg;
@@ -200,6 +200,9 @@ enqueue(int argc, char *argv[], FILE *ofp)
 		case 'N':
 			msg.dsn_notify = optarg;
 			break;
+		case 'r':
+			fake_from = optarg;
+			break;
 		case 'R':
 			msg.dsn_ret = optarg;
 			break;
