@@ -1,4 +1,4 @@
-/*	$OpenBSD: lka_session.c,v 1.66 2014/04/19 12:55:23 gilles Exp $	*/
+/*	$OpenBSD: lka_session.c,v 1.79 2015/12/28 22:08:30 jung Exp $	*/
 
 /*
  * Copyright (c) 2011 Gilles Chehade <gilles@poolp.org>
@@ -70,7 +70,6 @@ static void lka_submit(struct lka_session *, struct rule *,
 static void lka_resume(struct lka_session *);
 static size_t lka_expand_format(char *, size_t, const struct envelope *,
     const struct userinfo *);
-static void mailaddr_to_username(const struct mailaddr *, char *, size_t);
 
 static int mod_lowercase(char *, size_t);
 static int mod_uppercase(char *, size_t);
@@ -324,9 +323,8 @@ lka_expand(struct lka_session *lks, struct rule *rule, struct expandnode *xn)
 			 * we eventually strip the '+'-part before lookup.
 			 */
 			maddr = xn->u.mailaddr;
-			mailaddr_to_username(&xn->u.mailaddr, maddr.user,
+			xlowercase(maddr.user, xn->u.mailaddr.user,
 			    sizeof maddr.user);
-
 			r = aliases_virtual_get(&lks->expand, &maddr);
 			if (r == -1) {
 				lks->error = LKA_TEMPFAIL;
@@ -345,8 +343,8 @@ lka_expand(struct lka_session *lks, struct rule *rule, struct expandnode *xn)
 			lks->expand.alias = 1;
 			memset(&node, 0, sizeof node);
 			node.type = EXPAND_USERNAME;
-			mailaddr_to_username(&xn->u.mailaddr, node.u.user,
-				sizeof node.u.user);
+			xlowercase(node.u.user, xn->u.mailaddr.user,
+			    sizeof node.u.user);
 			node.mapping = rule->r_mapping;
 			node.userbase = rule->r_userbase;
 			expand_insert(&lks->expand, &node);
@@ -385,16 +383,6 @@ lka_expand(struct lka_session *lks, struct rule *rule, struct expandnode *xn)
 		if ((tag = strchr(xn->u.user, TAG_CHAR)) != NULL)
 			*tag++ = '\0';
 
-		/* A username should not exceed the size of a system user */
-		/*
-		if (strlen(xn->u.user) >= sizeof fwreq.user) {
-			log_trace(TRACE_EXPAND, "expand: lka_expand: "
-			    "user-part too long to be a system user");
-			lks->error = LKA_PERMFAIL;
-			break;
-		}
-		*/
-
 		r = table_lookup(rule->r_userbase, NULL, xn->u.user, K_USERINFO, &lk);
 		if (r == -1) {
 			log_trace(TRACE_EXPAND, "expand: lka_expand: "
@@ -658,41 +646,41 @@ lka_expand_token(char *dest, size_t len, const char *token,
 	}
 
 	/* token -> expanded token */
-	if (! strcasecmp("sender", rtoken)) {
+	if (!strcasecmp("sender", rtoken)) {
 		if (snprintf(tmp, sizeof tmp, "%s@%s",
-			ep->sender.user, ep->sender.domain) <= 0)
+			ep->sender.user, ep->sender.domain) >= (int)sizeof tmp)
 			return 0;
 		string = tmp;
 	}
-	else if (! strcasecmp("dest", rtoken)) {
+	else if (!strcasecmp("dest", rtoken)) {
 		if (snprintf(tmp, sizeof tmp, "%s@%s",
-			ep->dest.user, ep->dest.domain) <= 0)
+			ep->dest.user, ep->dest.domain) >= (int)sizeof tmp)
 			return 0;
 		string = tmp;
 	}
-	else if (! strcasecmp("rcpt", rtoken)) {
+	else if (!strcasecmp("rcpt", rtoken)) {
 		if (snprintf(tmp, sizeof tmp, "%s@%s",
-			ep->rcpt.user, ep->rcpt.domain) <= 0)
+			ep->rcpt.user, ep->rcpt.domain) >= (int)sizeof tmp)
 			return 0;
 		string = tmp;
 	}
-	else if (! strcasecmp("sender.user", rtoken))
+	else if (!strcasecmp("sender.user", rtoken))
 		string = ep->sender.user;
-	else if (! strcasecmp("sender.domain", rtoken))
+	else if (!strcasecmp("sender.domain", rtoken))
 		string = ep->sender.domain;
-	else if (! strcasecmp("user.username", rtoken))
+	else if (!strcasecmp("user.username", rtoken))
 		string = ui->username;
-	else if (! strcasecmp("user.directory", rtoken)) {
+	else if (!strcasecmp("user.directory", rtoken)) {
 		string = ui->directory;
 		replace = 0;
 	}
-	else if (! strcasecmp("dest.user", rtoken))
+	else if (!strcasecmp("dest.user", rtoken))
 		string = ep->dest.user;
-	else if (! strcasecmp("dest.domain", rtoken))
+	else if (!strcasecmp("dest.domain", rtoken))
 		string = ep->dest.domain;
-	else if (! strcasecmp("rcpt.user", rtoken))
+	else if (!strcasecmp("rcpt.user", rtoken))
 		string = ep->rcpt.user;
-	else if (! strcasecmp("rcpt.domain", rtoken))
+	else if (!strcasecmp("rcpt.domain", rtoken))
 		string = ep->rcpt.domain;
 	else
 		return 0;
@@ -709,12 +697,12 @@ lka_expand_token(char *dest, size_t len, const char *token,
 			if ((sep = strchr(mods, '|')) != NULL)
 				*sep++ = '\0';
 			for (i = 0; (size_t)i < nitems(token_modifiers); ++i) {
-				if (! strcasecmp(token_modifiers[i].name, mods)) {
+				if (!strcasecmp(token_modifiers[i].name, mods)) {
 					if (token_modifiers[i].f == NULL) {
 						raw = 1;
 						break;
 					}
-					if (! token_modifiers[i].f(tmp, sizeof tmp))
+					if (!token_modifiers[i].f(tmp, sizeof tmp))
 						return 0; /* modifier error */
 					break;
 				}
@@ -723,8 +711,8 @@ lka_expand_token(char *dest, size_t len, const char *token,
 				return 0; /* modifier not found */
 		} while ((mods = sep) != NULL);
 	}
-	
-	if (! raw && replace)
+
+	if (!raw && replace)
 		for (i = 0; (size_t)i < strlen(tmp); ++i)
 			if (strchr(MAILADDR_ESCAPE, tmp[i]))
 				tmp[i] = ':';
@@ -857,30 +845,24 @@ lka_expand_format(char *buf, size_t len, const struct envelope *ep,
 	return ret;
 }
 
-static void
-mailaddr_to_username(const struct mailaddr *maddr, char *dst, size_t len)
-{
-	xlowercase(dst, maddr->user, len);
-}
-
-static int 
+static int
 mod_lowercase(char *buf, size_t len)
 {
 	char tmp[EXPAND_BUFFER];
 
-	if (! lowercase(tmp, buf, sizeof tmp))
+	if (!lowercase(tmp, buf, sizeof tmp))
 		return 0;
 	if (strlcpy(buf, tmp, len) >= len)
 		return 0;
 	return 1;
 }
 
-static int 
+static int
 mod_uppercase(char *buf, size_t len)
 {
 	char tmp[EXPAND_BUFFER];
 
-	if (! uppercase(tmp, buf, sizeof tmp))
+	if (!uppercase(tmp, buf, sizeof tmp))
 		return 0;
 	if (strlcpy(buf, tmp, len) >= len)
 		return 0;
