@@ -1,4 +1,4 @@
-/*	$OpenBSD: mda.c,v 1.117 2016/02/02 05:45:27 sunil Exp $	*/
+/*	$OpenBSD: mda.c,v 1.119 2016/05/22 16:31:21 gilles Exp $	*/
 
 /*
  * Copyright (c) 2008 Gilles Chehade <gilles@poolp.org>
@@ -436,7 +436,7 @@ mda_imsg(struct mproc *p, struct imsg *imsg)
 			    "for session %016"PRIx64 " evpid %016"PRIx64,
 			    imsg->fd, s->id, s->evp->id);
 
-			io_set_blocking(imsg->fd, 0);
+			io_set_nonblocking(imsg->fd);
 			io_init(&s->io, imsg->fd, s, mda_io, &s->iobuf);
 			io_set_write(&s->io);
 			return;
@@ -791,9 +791,9 @@ mda_log(const struct mda_envelope *evp, const char *prefix, const char *status)
 	else
 		method = "???";
 
-	log_info("delivery: %s for %016" PRIx64 ": from=<%s>, to=<%s>, "
-	    "%suser=%s, method=%s, delay=%s, stat=%s",
-	    prefix,
+	log_info("%016"PRIx64" mda event=delivery evpid=%016" PRIx64 " from=<%s> to=<%s> "
+	    "%suser=%s method=%s delay=%s result=%s stat=%s",
+	    (uint64_t)0,
 	    evp->id,
 	    evp->sender ? evp->sender : "",
 	    evp->dest,
@@ -801,6 +801,7 @@ mda_log(const struct mda_envelope *evp, const char *prefix, const char *status)
 	    evp->user,
 	    method,
 	    duration_to_text(time(NULL) - evp->creation),
+	    prefix,
 	    status);
 }
 
