@@ -1,4 +1,4 @@
-/*	$OpenBSD: mta.c,v 1.200 2016/01/14 18:56:55 mmcc Exp $	*/
+/*	$OpenBSD: mta.c,v 1.201 2016/05/22 16:31:21 gilles Exp $	*/
 
 /*
  * Copyright (c) 2008 Pierre-Yves Ritschard <pyr@openbsd.org>
@@ -1612,18 +1612,18 @@ static void
 mta_log(const struct mta_envelope *evp, const char *prefix, const char *source,
     const char *relay, const char *status)
 {
-	log_info("relay: %s for %016" PRIx64 ": session=%016"PRIx64", "
-	    "from=<%s>, to=<%s>, rcpt=<%s>, source=%s, "
-	    "relay=%s, delay=%s, stat=%s",
-	    prefix,
-	    evp->id,
+	log_info("%016"PRIx64" mta event=delivery evpid=%016"PRIx64" "
+	    "from=<%s> to=<%s> rcpt=<%s> source=%s "
+	    "relay=%s delay=%s result=%s stat=%s",
 	    evp->session,
+	    evp->id,
 	    evp->task->sender,
 	    evp->dest,
 	    evp->rcpt ? evp->rcpt : "-",
 	    source ? source : "-",
 	    relay,
 	    duration_to_text(time(NULL) - evp->creation),
+	    prefix,
 	    status);
 }
 
