@@ -1,4 +1,4 @@
-/*	$OpenBSD$	*/
+/*	$OpenBSD: queue_backend.c,v 1.62 2016/02/04 12:46:28 eric Exp $	*/
 
 /*
  * Copyright (c) 2011 Gilles Chehade <gilles@poolp.org>
@@ -29,6 +29,7 @@
 #include <errno.h>
 #include <event.h>
 #include <fcntl.h>
+#include <grp.h>
 #include <imsg.h>
 #include <limits.h>
 #include <inttypes.h>
@@ -118,12 +119,17 @@ int
 queue_init(const char *name, int server)
 {
 	struct passwd	*pwq;
+	struct group	*gr;
 	int		 r;
 
 	pwq = getpwnam(SMTPD_QUEUE_USER);
 	if (pwq == NULL)
 		errx(1, "unknown user %s", SMTPD_QUEUE_USER);
 
+	gr = getgrnam(SMTPD_QUEUE_GROUP);
+	if (gr == NULL)
+		errx(1, "unknown group %s", SMTPD_QUEUE_GROUP);
+
 	tree_init(&evpcache_tree);
 	TAILQ_INIT(&evpcache_list);
 
@@ -139,11 +145,9 @@ queue_init(const char *name, int server)
 	if (server) {
 		if (ckdir(PATH_SPOOL, 0711, 0, 0, 1) == 0)
 			errx(1, "error in spool directory setup");
-		if (ckdir(PATH_SPOOL PATH_OFFLINE, 01777, 0, 0, 1) == 0)
+		if (ckdir(PATH_SPOOL PATH_OFFLINE, 0770, 0, gr->gr_gid, 1) == 0)
 			errx(1, "error in offline directory setup");
-		if (ckdir_quiet(PATH_SPOOL PATH_PURGE, 0700, pwq->pw_uid, 0))
-			chmod(PATH_SPOOL PATH_PURGE, 0750);
-		if (ckdir(PATH_SPOOL PATH_PURGE, 0750, pwq->pw_uid, 0, 1) == 0)
+		if (ckdir(PATH_SPOOL PATH_PURGE, 0700, pwq->pw_uid, 0, 1) == 0)
 			errx(1, "error in purge directory setup");
 
 		mvpurge(PATH_SPOOL PATH_TEMPORARY, PATH_SPOOL PATH_PURGE);
@@ -188,6 +192,8 @@ int
 queue_message_delete(uint32_t msgid)
 {
 	char	msgpath[PATH_MAX];
+	uint64_t evpid;
+	void   *iter;
 	int	r;
 
 	profile_enter("queue_message_delete");
@@ -198,6 +204,17 @@ queue_message_delete(uint32_t msgid)
 	queue_message_path(msgid, msgpath, sizeof(msgpath));
 	unlink(msgpath);
 
+	/* remove remaining envelopes from the cache if any (on rollback) */
+	evpid = msgid_to_evpid(msgid);
+	for (;;) {
+		iter = NULL;
+		if (!tree_iterfrom(&evpcache_tree, &iter, evpid, &evpid, NULL))
+			break;
+		if (evpid_to_msgid(evpid) != msgid)
+			break;
+		queue_envelope_cache_del(evpid);
+	}
+
 	log_trace(TRACE_QUEUE,
 	    "queue-backend: queue_message_delete(%08"PRIx32") -> %d", msgid, r);
 
@@ -223,7 +240,7 @@ queue_message_commit(uint32_t msgid)
 		ofp = fopen(tmppath, "w+");
 		if (ifp == NULL || ofp == NULL)
 			goto err;
-		if (! compress_file(ifp, ofp))
+		if (!compress_file(ifp, ofp))
 			goto err;
 		fclose(ifp);
 		fclose(ofp);
@@ -246,7 +263,7 @@ queue_message_commit(uint32_t msgid)
 		ofp = fopen(tmppath, "w+");
 		if (ifp == NULL || ofp == NULL)
 			goto err;
-		if (! crypto_encrypt_file(ifp, ofp))
+		if (!crypto_encrypt_file(ifp, ofp))
 			goto err;
 		fclose(ifp);
 		fclose(ofp);
@@ -307,7 +324,7 @@ queue_message_uncorrupt(uint32_t msgid)
 int
 queue_message_fd_r(uint32_t msgid)
 {
-	int	fdin, fdout = -1, fd = -1;
+	int	fdin = -1, fdout = -1, fd = -1;
 	FILE	*ifp = NULL;
 	FILE	*ofp = NULL;
 
@@ -334,7 +351,7 @@ queue_message_fd_r(uint32_t msgid)
 		if ((ofp = fdopen(fdout, "w+")) == NULL)
 			goto err;
 
-		if (! crypto_decrypt_file(ifp, ofp))
+		if (!crypto_decrypt_file(ifp, ofp))
 			goto err;
 
 		fclose(ifp);
@@ -357,7 +374,7 @@ queue_message_fd_r(uint32_t msgid)
 		if ((ofp = fdopen(fdout, "w+")) == NULL)
 			goto err;
 
-		if (! uncompress_file(ifp, ofp))
+		if (!uncompress_file(ifp, ofp))
 			goto err;
 
 		fclose(ifp);
@@ -663,7 +680,7 @@ queue_message_walk(struct envelope *ep, uint32_t msgid, int *done, void **data)
 			/*
 			 * do not cache the envelope here, while discovering
 			 * envelopes one could re-run discover on already
-			 * scheduled envelopes which leads to triggering of 
+			 * scheduled envelopes which leads to triggering of
 			 * strict checks in caching. Envelopes could anyway
 			 * be loaded from backend if it isn't cached.
 			 */
