@@ -1,4 +1,4 @@
-/*	$OpenBSD: queue_fs.c,v 1.5 2014/04/19 13:48:57 gilles Exp $	*/
+/*	$OpenBSD: queue_fs.c,v 1.14 2015/12/30 11:40:30 jung Exp $	*/
 
 /*
  * Copyright (c) 2011 Gilles Chehade <gilles@poolp.org>
@@ -19,7 +19,6 @@
 #include "includes.h"
 
 #include <sys/types.h>
-#include <sys/param.h>
 #if HAVE_SYS_MOUNT_H
 #include <sys/mount.h>
 #endif
@@ -41,7 +40,6 @@
 #include <imsg.h>
 #include <inttypes.h>
 #include <libgen.h>
-#include <limits.h>
 #include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -58,8 +56,9 @@
 #define PATH_EVPTMP		PATH_INCOMING "/envelope.tmp"
 #define PATH_MESSAGE		"/message"
 
-/* minimum number of free blocks on filesystem */
-#define	MINFREEBLOCKS		100
+/* percentage of remaining space / inodes required to accept new messages */
+#define	MINSPACE		5
+#define	MININODES		5
 
 struct qwalk {
 	FTS	*fts;
@@ -88,7 +87,7 @@ queue_fs_message_create(uint32_t *msgid)
 	char		rootdir[PATH_MAX];
 	struct stat	sb;
 
-	if (! fsqueue_check_space())
+	if (!fsqueue_check_space())
 		return 0;
 
 again:
@@ -174,9 +173,6 @@ queue_fs_message_commit(uint32_t msgid, const char *path)
 		return 0;
 	}
 
-	/* best effort */
-	sync();
-
 	return 1;
 }
 
@@ -269,7 +265,7 @@ queue_fs_message_uncorrupt(uint32_t msgid)
 		return (0);
 	}
 
-	if (! bsnprintf(bucketdir, sizeof bucketdir, "%s/%02x", PATH_QUEUE,
+	if (!bsnprintf(bucketdir, sizeof bucketdir, "%s/%02x", PATH_QUEUE,
 	    (msgid & 0xff000000) >> 24)) {
 		log_warnx("warn: queue-fs: path too long");
 		return (0);
@@ -305,7 +301,7 @@ queue_fs_envelope_create(uint32_t msgid, const char *buf, size_t len,
 		log_warnx("warn: queue-fs: msgid=0, evpid=%016"PRIx64, *evpid);
 		goto done;
 	}
-	
+
 	fsqueue_message_incoming_path(msgid, path, sizeof(path));
 	if (stat(path, &sb) == -1)
 		queued = 1;
@@ -414,7 +410,7 @@ queue_fs_message_walk(uint64_t *evpid, char *buf, size_t len,
 	if (*done)
 		return (-1);
 
-	if (! bsnprintf(path, sizeof path, "%s/%02x/%08x",
+	if (!bsnprintf(path, sizeof path, "%s/%02x/%08x",
 	    PATH_QUEUE, (msgid  & 0xff000000) >> 24, msgid))
 		fatalx("queue_fs_message_walk: path does not fit buffer");
 
@@ -434,7 +430,8 @@ queue_fs_message_walk(uint64_t *evpid, char *buf, size_t len,
 			continue;
 
 		/* ignore files other than envelopes */
-		if (strlen(dp->d_name) != 16 || strncmp(dp->d_name, msgid_str, 8))
+		if (strlen(dp->d_name) != 16 ||
+		    strncmp(dp->d_name, msgid_str, 8))
 			continue;
 
 		tmp = NULL;
@@ -499,7 +496,10 @@ queue_fs_envelope_walk(uint64_t *evpid, char *buf, size_t len)
 static int
 fsqueue_check_space(void)
 {
+#ifdef __OpenBSD__
 	struct statfs	buf;
+	uint64_t	used;
+	uint64_t	total;
 
 	if (statfs(PATH_QUEUE, &buf) < 0) {
 		log_warn("warn: queue-fs: statfs");
@@ -515,18 +515,39 @@ fsqueue_check_space(void)
 	    (int64_t)buf.f_bfree == -1 || (int64_t)buf.f_ffree == -1)
 		return 1;
 
-	if (buf.f_bavail < MINFREEBLOCKS) {
-		log_warnx("warn: disk space running low, temporarily rejecting messages");
+	used = buf.f_blocks - buf.f_bfree;
+	total = buf.f_bavail + used;
+	if (total != 0)
+		used = (float)used / (float)total * 100;
+	else
+		used = 100;
+	if (100 - used < MINSPACE) {
+		log_warnx("warn: not enough disk space: %llu%% left",
+		    (unsigned long long) 100 - used);
+		log_warnx("warn: temporarily rejecting messages");
 		return 0;
 	}
 
+	used = buf.f_files - buf.f_ffree;
+	total = buf.f_favail + used;
+	if (total != 0)
+		used = (float)used / (float)total * 100;
+	else
+		used = 100;
+	if (100 - used < MININODES) {
+		log_warnx("warn: not enough inodes: %llu%% left",
+		    (unsigned long long) 100 - used);
+		log_warnx("warn: temporarily rejecting messages");
+		return 0;
+	}
+#endif
 	return 1;
 }
 
 static void
 fsqueue_envelope_path(uint64_t evpid, char *buf, size_t len)
 {
-	if (! bsnprintf(buf, len, "%s/%02x/%08x/%016" PRIx64,
+	if (!bsnprintf(buf, len, "%s/%02x/%08x/%016" PRIx64,
 		PATH_QUEUE,
 		(evpid_to_msgid(evpid) & 0xff000000) >> 24,
 		evpid_to_msgid(evpid),
@@ -537,7 +558,7 @@ fsqueue_envelope_path(uint64_t evpid, char *buf, size_t len)
 static void
 fsqueue_envelope_incoming_path(uint64_t evpid, char *buf, size_t len)
 {
-	if (! bsnprintf(buf, len, "%s/%08x/%016" PRIx64,
+	if (!bsnprintf(buf, len, "%s/%08x/%016" PRIx64,
 		PATH_INCOMING,
 		evpid_to_msgid(evpid),
 		evpid))
@@ -603,7 +624,7 @@ tempfail:
 static void
 fsqueue_message_path(uint32_t msgid, char *buf, size_t len)
 {
-	if (! bsnprintf(buf, len, "%s/%02x/%08x",
+	if (!bsnprintf(buf, len, "%s/%02x/%08x",
 		PATH_QUEUE,
 		(msgid & 0xff000000) >> 24,
 		msgid))
@@ -613,7 +634,7 @@ fsqueue_message_path(uint32_t msgid, char *buf, size_t len)
 static void
 fsqueue_message_corrupt_path(uint32_t msgid, char *buf, size_t len)
 {
-	if (! bsnprintf(buf, len, "%s/%08x",
+	if (!bsnprintf(buf, len, "%s/%08x",
 		PATH_CORRUPT,
 		msgid))
 		fatalx("fsqueue_message_corrupt_path: path does not fit buffer");
@@ -622,7 +643,7 @@ fsqueue_message_corrupt_path(uint32_t msgid, char *buf, size_t len)
 static void
 fsqueue_message_incoming_path(uint32_t msgid, char *buf, size_t len)
 {
-	if (! bsnprintf(buf, len, "%s/%08x",
+	if (!bsnprintf(buf, len, "%s/%08x",
 		PATH_INCOMING,
 		msgid))
 		fatalx("fsqueue_message_incoming_path: path does not fit buffer");
