@@ -1,4 +1,4 @@
-/*	$OpenBSD: rfc2822.c,v 1.7 2016/02/04 22:35:17 eric Exp $	*/
+/*	$OpenBSD: rfc2822.c,v 1.8 2016/08/20 20:47:08 eric Exp $	*/
 
 /*
  * Copyright (c) 2014 Gilles Chehade <gilles@poolp.org>
@@ -52,6 +52,9 @@ header_callback(struct rfc2822_parser *rp)
 	struct rfc2822_hdr_cb		*hdr_cb;
 	struct rfc2822_hdr_miss_cb	*hdr_miss_cb;
 
+	if (!rp->in_hdr)
+		goto end;
+
 	TAILQ_FOREACH(hdr_cb, &rp->hdr_cb, next)
 	    if (strcasecmp(hdr_cb->name, rp->header.name) == 0) {
 		    hdr_cb->func(&rp->header, hdr_cb->arg);
@@ -140,6 +143,7 @@ rfc2822_parser_init(struct rfc2822_parser *rp)
 	TAILQ_INIT(&rp->header.lines);
 	rfc2822_header_default_callback(rp, hdr_dflt_cb, NULL);
 	rfc2822_body_callback(rp, body_dflt_cb, NULL);
+	rfc2822_parser_reset(rp);
 }
 
 void
@@ -149,6 +153,8 @@ rfc2822_parser_flush(struct rfc2822_parser *rp)
 		return;
 
 	header_callback(rp);
+
+	missing_headers_callback(rp);
 }
 
 void
