@@ -1,4 +1,4 @@
-/*	$OpenBSD: smtp_session.c,v 1.268 2016/02/05 19:15:15 jung Exp $	*/
+/*	$OpenBSD: smtp_session.c,v 1.284 2016/07/22 12:12:29 eric Exp $	*/
 
 /*
  * Copyright (c) 2008 Gilles Chehade <gilles@poolp.org>
@@ -79,6 +79,7 @@ enum session_flags {
 	SF_BADINPUT		= 0x0080,
 	SF_FILTERCONN		= 0x0100,
 	SF_FILTERDATA		= 0x0200,
+	SF_FILTERTX		= 0x0400,
 };
 
 enum message_flags {
@@ -112,6 +113,30 @@ struct smtp_rcpt {
 	size_t			 destcount;
 };
 
+struct smtp_tx {
+	struct smtp_session	*session;
+	uint32_t		 msgid;
+
+	struct envelope		 evp;
+	size_t			 rcptcount;
+	size_t			 destcount;
+	TAILQ_HEAD(, smtp_rcpt)	 rcpts;
+
+	size_t			 datain;
+	size_t			 odatalen;
+	struct iobuf		 obuf;
+	struct io		 oev;
+	int			 hdrdone;
+	int			 rcvcount;
+	int			 dataeom;
+
+	int			 msgflags;
+	int			 msgcode;
+
+	int			 skiphdr;
+	struct rfc2822_parser	 rfc2822_parser;
+};
+
 struct smtp_session {
 	uint64_t		 id;
 	struct iobuf		 iobuf;
@@ -130,28 +155,10 @@ struct smtp_session {
 	char			 cmd[LINE_MAX];
 	char			 username[SMTPD_MAXMAILADDRSIZE];
 
-	struct envelope		 evp;
-
 	size_t			 mailcount;
-
-	int			 msgflags;
-	int			 msgcode;
-	size_t			 rcptcount;
-	size_t			 destcount;
-	TAILQ_HEAD(, smtp_rcpt)	 rcpts;
-
-	size_t			 datain;
-	size_t			 odatalen;
-	struct iobuf		 obuf;
-	struct io		 oev;
-	int			 hdrdone;
-	int			 rcvcount;
-	int			 dataeom;
-
-	int			 skiphdr;
 	struct event		 pause;
 
-	struct rfc2822_parser	 rfc2822_parser;
+	struct smtp_tx		*tx;
 };
 
 #define ADVERTISE_TLS(s) \
@@ -180,7 +187,6 @@ static int smtp_parse_rcpt_args(struct smtp_session *, char *);
 static void smtp_rfc4954_auth_plain(struct smtp_session *, char *);
 static void smtp_rfc4954_auth_login(struct smtp_session *, char *);
 static void smtp_message_end(struct smtp_session *);
-static void smtp_message_reset(struct smtp_session *, int);
 static int smtp_message_printf(struct smtp_session *, const char *, ...);
 static void smtp_free(struct smtp_session *, const char *);
 static const char *smtp_strstate(int);
@@ -189,6 +195,9 @@ static uint8_t dsn_notify_str_to_uint8(const char *);
 static void smtp_auth_failure_pause(struct smtp_session *);
 static void smtp_auth_failure_resume(int, short, void *);
 
+static int  smtp_tx(struct smtp_session *);
+static void smtp_tx_free(struct smtp_tx *);
+
 static void smtp_queue_create_message(struct smtp_session *);
 static void smtp_queue_open_message(struct smtp_session *);
 static void smtp_queue_commit(struct smtp_session *);
@@ -197,8 +206,9 @@ static void smtp_queue_rollback(struct smtp_session *);
 static void smtp_filter_connect(struct smtp_session *, struct sockaddr *);
 static void smtp_filter_rset(struct smtp_session *);
 static void smtp_filter_disconnect(struct smtp_session *);
-static void smtp_filter_commit(struct smtp_session *);
-static void smtp_filter_rollback(struct smtp_session *);
+static void smtp_filter_tx_begin(struct smtp_session *);
+static void smtp_filter_tx_commit(struct smtp_session *);
+static void smtp_filter_tx_rollback(struct smtp_session *);
 static void smtp_filter_eom(struct smtp_session *);
 static void smtp_filter_helo(struct smtp_session *);
 static void smtp_filter_mail(struct smtp_session *);
@@ -539,7 +549,7 @@ header_masquerade_callback(const struct rfc2822_header *hdr, void *arg)
 			if (l->buffer[i] == ',' && !escape && !quote && !comment) {
 				if (!skip && j + strlen(s->listener->hostname) + 1 < sizeof buffer) {
 					header_append_domain_buffer(buffer, s->listener->hostname, sizeof buffer);
-					header_address_rewrite_buffer(buffer, mailaddr_to_text(&s->evp.sender),
+					header_address_rewrite_buffer(buffer, mailaddr_to_text(&s->tx->evp.sender),
 					    sizeof buffer);
 				}
 				if (smtp_message_printf(s, "%s,", buffer) == -1)
@@ -585,7 +595,7 @@ header_masquerade_callback(const struct rfc2822_header *hdr, void *arg)
 	if (buffer[0]) {
 		if (j + strlen(s->listener->hostname) + 1 < sizeof buffer) {
 			header_append_domain_buffer(buffer, s->listener->hostname, sizeof buffer);
-			header_address_rewrite_buffer(buffer, mailaddr_to_text(&s->evp.sender),
+			header_address_rewrite_buffer(buffer, mailaddr_to_text(&s->tx->evp.sender),
 			    sizeof buffer);
 		}
 		smtp_message_printf(s, "%s", buffer);
@@ -639,11 +649,11 @@ smtp_session(struct listener *listener, int sock,
 
 	if ((s = calloc(1, sizeof(*s))) == NULL)
 		return (-1);
+
 	if (iobuf_init(&s->iobuf, LINE_MAX, LINE_MAX) == -1) {
 		free(s);
 		return (-1);
 	}
-	TAILQ_INIT(&s->rcpts);
 
 	s->id = generate_uid();
 	s->listener = listener;
@@ -651,7 +661,6 @@ smtp_session(struct listener *listener, int sock,
 	io_init(&s->io, sock, s, smtp_io, &s->iobuf);
 	io_set_timeout(&s->io, SMTPD_SESSION_TIMEOUT * 1000);
 	io_set_write(&s->io);
-	io_init(&s->oev, -1, s, NULL, NULL); /* initialise 'sock', but not to 0 */
 
 	s->state = STATE_NEW;
 	s->phase = PHASE_INIT;
@@ -662,28 +671,6 @@ smtp_session(struct listener *listener, int sock,
 	    "[hostname=%s, port=%d, tag=%s]", s, listener,
 	    listener->hostname, ntohs(listener->port), listener->tag);
 
-	/* Setup parser and callbacks before smtp_connected() can be called */
-	rfc2822_parser_init(&s->rfc2822_parser);
-	rfc2822_header_default_callback(&s->rfc2822_parser,
-	    header_default_callback, s);
-	rfc2822_header_callback(&s->rfc2822_parser, "bcc",
-	    header_bcc_callback, s);
-	rfc2822_header_callback(&s->rfc2822_parser, "from",
-	    header_domain_append_callback, s);
-	rfc2822_header_callback(&s->rfc2822_parser, "to",
-	    header_domain_append_callback, s);
-	rfc2822_header_callback(&s->rfc2822_parser, "cc",
-	    header_domain_append_callback, s);
-	rfc2822_body_callback(&s->rfc2822_parser,
-	    dataline_callback, s);
-
-	if (hostname || listener->local) {
-		rfc2822_missing_header_callback(&s->rfc2822_parser, "date",
-		    header_missing_callback, s);
-		rfc2822_missing_header_callback(&s->rfc2822_parser, "message-id",
-		    header_missing_callback, s);
-	}
-
 	/* For local enqueueing, the hostname is already set */
 	if (hostname) {
 		s->flags |= SF_AUTHENTICATED;
@@ -750,15 +737,19 @@ smtp_session_imsg(struct mproc *p, struct imsg *imsg)
 
 			/* sender check passed, override From callback if masquerading */
 			if (s->listener->flags & F_MASQUERADE)
-				rfc2822_header_callback(&s->rfc2822_parser, "from",
+				rfc2822_header_callback(&s->tx->rfc2822_parser, "from",
 				    header_masquerade_callback, s);
 			break;
 
 		case LKA_PERMFAIL:
+			smtp_filter_tx_rollback(s);
+			smtp_tx_free(s->tx);
 			smtp_reply(s, "%d %s", 530, "Sender rejected");
 			io_reload(&s->io);
 			break;
 		case LKA_TEMPFAIL:
+			smtp_filter_tx_rollback(s);
+			smtp_tx_free(s->tx);
 			smtp_reply(s, "421 %s: Temporary Error",
 			    esc_code(ESC_STATUS_TEMPFAIL, ESC_OTHER_MAIL_SYSTEM_STATUS));
 			io_reload(&s->io);
@@ -805,12 +796,15 @@ smtp_session_imsg(struct mproc *p, struct imsg *imsg)
 		s = tree_xpop(&wait_queue_msg, reqid);
 		if (success) {
 			m_get_msgid(&m, &msgid);
-			s->evp.id = msgid_to_evpid(msgid);
-			s->rcptcount = 0;
+			s->tx->msgid = msgid;
+			s->tx->evp.id = msgid_to_evpid(msgid);
+			s->tx->rcptcount = 0;
 			s->phase = PHASE_TRANSACTION;
 			smtp_reply(s, "250 %s: Ok",
 			    esc_code(ESC_STATUS_OK, ESC_OTHER_STATUS));
 		} else {
+			smtp_filter_tx_rollback(s);
+			smtp_tx_free(s->tx);
 			smtp_reply(s, "421 %s: Temporary Error",
 			    esc_code(ESC_STATUS_TEMPFAIL, ESC_OTHER_MAIL_SYSTEM_STATUS));
 			smtp_enter_state(s, STATE_QUIT);
@@ -849,10 +843,10 @@ smtp_session_imsg(struct mproc *p, struct imsg *imsg)
 		s = tree_xget(&wait_lka_rcpt, reqid);
 		if (success) {
 			m_get_evpid(&m, &evpid);
-			s->destcount++;
+			s->tx->destcount++;
 		}
 		else
-			s->msgflags |= MF_QUEUE_ENVELOPE_FAIL;
+			s->tx->msgflags |= MF_QUEUE_ENVELOPE_FAIL;
 		m_end(&m);
 		return;
 
@@ -864,7 +858,7 @@ smtp_session_imsg(struct mproc *p, struct imsg *imsg)
 		if (!success)
 			fatalx("commit evp failed: not supposed to happen");
 		s = tree_xpop(&wait_lka_rcpt, reqid);
-		if (s->msgflags & MF_QUEUE_ENVELOPE_FAIL) {
+		if (s->tx->msgflags & MF_QUEUE_ENVELOPE_FAIL) {
 			/*
 			 * If an envelope failed, we can't cancel the last
 			 * RCPT only so we must cancel the whole transaction
@@ -876,12 +870,12 @@ smtp_session_imsg(struct mproc *p, struct imsg *imsg)
 		}
 		else {
 			rcpt = xcalloc(1, sizeof(*rcpt), "smtp_rcpt");
-			rcpt->destcount = s->destcount;
-			rcpt->maddr = s->evp.rcpt;
-			TAILQ_INSERT_TAIL(&s->rcpts, rcpt, entry);
+			rcpt->destcount = s->tx->destcount;
+			rcpt->maddr = s->tx->evp.rcpt;
+			TAILQ_INSERT_TAIL(&s->tx->rcpts, rcpt, entry);
 
-			s->destcount = 0;
-			s->rcptcount++;
+			s->tx->destcount = 0;
+			s->tx->rcptcount++;
 			smtp_reply(s, "250 %s %s: Recipient ok",
 			    esc_code(ESC_STATUS_OK, ESC_DESTINATION_ADDRESS_VALID),
 			    esc_description(ESC_DESTINATION_ADDRESS_VALID));
@@ -896,7 +890,8 @@ smtp_session_imsg(struct mproc *p, struct imsg *imsg)
 		m_end(&m);
 		s = tree_xpop(&wait_queue_commit, reqid);
 		if (!success) {
-			smtp_filter_rollback(s);
+			smtp_filter_tx_rollback(s);
+			smtp_tx_free(s->tx);
 			smtp_reply(s, "421 %s: Temporary failure",
 			    esc_code(ESC_STATUS_TEMPFAIL, ESC_OTHER_MAIL_SYSTEM_STATUS));
 			smtp_enter_state(s, STATE_QUIT);
@@ -904,31 +899,29 @@ smtp_session_imsg(struct mproc *p, struct imsg *imsg)
 			return;
 		}
 
-		smtp_filter_commit(s);
+		smtp_filter_tx_commit(s);
 		smtp_reply(s, "250 %s: %08x Message accepted for delivery",
 		    esc_code(ESC_STATUS_OK, ESC_OTHER_STATUS),
-		    evpid_to_msgid(s->evp.id));
+		    s->tx->msgid);
 
-		TAILQ_FOREACH(rcpt, &s->rcpts, entry) {
-			log_info("smtp-in: Accepted message %08x "
-			    "on session %016"PRIx64
-			    ": from=<%s%s%s>, to=<%s%s%s>, size=%zu, ndest=%zu, proto=%s",
-			    evpid_to_msgid(s->evp.id),
+		TAILQ_FOREACH(rcpt, &s->tx->rcpts, entry) {
+			log_info("%016"PRIx64" smtp event=message msgid=%08x "
+			    "from=<%s%s%s> to=<%s%s%s> size=%zu ndest=%zu proto=%s",
 			    s->id,
-			    s->evp.sender.user,
-			    s->evp.sender.user[0] == '\0' ? "" : "@",
-			    s->evp.sender.domain,
+			    s->tx->msgid,
+			    s->tx->evp.sender.user,
+			    s->tx->evp.sender.user[0] == '\0' ? "" : "@",
+			    s->tx->evp.sender.domain,
 			    rcpt->maddr.user,
 			    rcpt->maddr.user[0] == '\0' ? "" : "@",
 			    rcpt->maddr.domain,
-			    s->odatalen,
+			    s->tx->odatalen,
 			    rcpt->destcount,
 			    s->flags & SF_EHLO ? "ESMTP" : "SMTP");
 		}
-
+		smtp_tx_free(s->tx);
 		s->mailcount++;
 		s->phase = PHASE_SETUP;
-		smtp_message_reset(s, 0);
 		smtp_enter_state(s, STATE_HELO);
 		io_reload(&s->io);
 		return;
@@ -942,21 +935,24 @@ smtp_session_imsg(struct mproc *p, struct imsg *imsg)
 		s = tree_xpop(&wait_parent_auth, reqid);
 		strnvis(user, s->username, sizeof user, VIS_WHITE | VIS_SAFE);
 		if (success == LKA_OK) {
-			log_info("smtp-in: Accepted authentication for user %s "
-			    "on session %016"PRIx64, user, s->id);
+			log_info("%016"PRIx64" smtp "
+			    "event=authentication user=%s result=ok",
+			    s->id, user);
 			s->flags |= SF_AUTHENTICATED;
 			smtp_reply(s, "235 %s: Authentication succeeded",
 			    esc_code(ESC_STATUS_OK, ESC_OTHER_STATUS));
 		}
 		else if (success == LKA_PERMFAIL) {
-			log_info("smtp-in: Authentication failed for user %s "
-			    "on session %016"PRIx64, user, s->id);
+			log_info("%016"PRIx64" smtp "
+			    "event=authentication user=%s result=permfail",
+			    s->id, user);
 			smtp_auth_failure_pause(s);
 			return;
 		}
 		else if (success == LKA_TEMPFAIL) {
-			log_info("smtp-in: Authentication temporarily failed "
-			    "for user %s on session %016"PRIx64, user, s->id);
+			log_info("%016"PRIx64" smtp "
+			    "event=authentication user=%s result=tempfail",
+			    s->id, user);
 			smtp_reply(s, "421 %s: Temporary failure",
 			    esc_code(ESC_STATUS_TEMPFAIL, ESC_OTHER_MAIL_SYSTEM_STATUS));
 		}
@@ -972,8 +968,8 @@ smtp_session_imsg(struct mproc *p, struct imsg *imsg)
 		s = tree_xpop(&wait_ssl_init, resp_ca_cert->reqid);
 
 		if (resp_ca_cert->status == CA_FAIL) {
-			log_info("smtp-in: Disconnecting session %016" PRIx64
-			    ": CA failure", s->id);
+			log_info("%016"PRIx64" smtp event=closed reason=ca-failure",
+			    s->id);
 			smtp_free(s, "CA failure");
 			return;
 		}
@@ -998,8 +994,9 @@ smtp_session_imsg(struct mproc *p, struct imsg *imsg)
 		if (resp_ca_vrfy->status == CA_OK)
 			s->flags |= SF_VERIFIED;
 		else if (s->listener->flags & F_TLS_VERIFY) {
-			log_info("smtp-in: Disconnecting session %016" PRIx64
-			    ": SSL certificate check failed", s->id);
+			log_info("%016"PRIx64" smtp "
+			    "event=closed reason=cert-check-failed",
+			    s->id);
 			smtp_free(s, "SSL certificate check failed");
 			return;
 		}
@@ -1035,8 +1032,9 @@ smtp_filter_response(uint64_t id, int query, int status, uint32_t code,
 
 	case QUERY_CONNECT:
 		if (status != FILTER_OK) {
-			log_info("smtp-in: Disconnecting session %016" PRIx64
-			    ": rejected by filter", s->id);
+			log_info("%016"PRIx64" smtp "
+			    "event=closed reason=filter-reject",
+			    s->id);
 			smtp_free(s, "rejected by filter");
 			return;
 		}
@@ -1074,7 +1072,7 @@ smtp_filter_response(uint64_t id, int query, int status, uint32_t code,
 		smtp_reply(s, "250%c%s Hello %s [%s], pleased to meet you",
 		    (s->flags & SF_EHLO) ? '-' : ' ',
 		    s->smtpname,
-		    s->evp.helo,
+		    s->helo,
 		    ss_to_text(&s->ss));
 
 		if (s->flags & SF_EHLO) {
@@ -1095,7 +1093,8 @@ smtp_filter_response(uint64_t id, int query, int status, uint32_t code,
 
 	case QUERY_MAIL:
 		if (status != FILTER_OK) {
-			smtp_filter_rollback(s);
+			smtp_filter_tx_rollback(s);
+			smtp_tx_free(s->tx);
 			code = code ? code : 530;
 			line = line ? line : "Sender rejected";
 			smtp_reply(s, "%d %s", code, line);
@@ -1109,7 +1108,7 @@ smtp_filter_response(uint64_t id, int query, int status, uint32_t code,
 			m_add_id(p_lka, s->id);
 			m_add_string(p_lka, s->listener->sendertable);
 			m_add_string(p_lka, s->username);
-			m_add_mailaddr(p_lka, &s->evp.sender);
+			m_add_mailaddr(p_lka, &s->tx->evp.sender);
 			m_close(p_lka);
 			tree_xset(&wait_lka_mail, s->id, s);
 		}
@@ -1128,7 +1127,7 @@ smtp_filter_response(uint64_t id, int query, int status, uint32_t code,
 
 		m_create(p_lka, IMSG_SMTP_EXPAND_RCPT, 0, 0, -1);
 		m_add_id(p_lka, s->id);
-		m_add_envelope(p_lka, &s->evp);
+		m_add_envelope(p_lka, &s->tx->evp);
 		m_close(p_lka);
 		tree_xset(&wait_lka_rcpt, s->id, s);
 		return;
@@ -1147,10 +1146,14 @@ smtp_filter_response(uint64_t id, int query, int status, uint32_t code,
 	case QUERY_EOM:
 		if (status != FILTER_OK) {
 			tree_pop(&wait_filter_data, s->id);
-			smtp_filter_rollback(s);
+			smtp_filter_tx_rollback(s);
+			smtp_queue_rollback(s);
+			smtp_tx_free(s->tx);
 			code = code ? code : 530;
 			line = line ? line : "Message rejected";
 			smtp_reply(s, "%d %s", code, line);
+			smtp_enter_state(s, STATE_HELO);
+			s->phase = PHASE_SETUP;
 			io_reload(&s->io);
 			return;
 		}
@@ -1180,29 +1183,29 @@ smtp_filter_fd(uint64_t id, int fd)
 		return;
 	}
 
-	iobuf_init(&s->obuf, 0, 0);
-	io_set_blocking(fd, 0);
-	io_init(&s->oev, fd, s, smtp_data_io, &s->obuf);
+	iobuf_init(&s->tx->obuf, 0, 0);
+	io_set_nonblocking(fd);
+	io_init(&s->tx->oev, fd, s, smtp_data_io, &s->tx->obuf);
 
-	iobuf_fqueue(&s->obuf, "Received: ");
+	iobuf_fqueue(&s->tx->obuf, "Received: ");
 	if (!(s->listener->flags & F_MASK_SOURCE)) {
-		iobuf_fqueue(&s->obuf, "from %s (%s [%s])",
-		    s->evp.helo,
+		iobuf_fqueue(&s->tx->obuf, "from %s (%s [%s])",
+		    s->helo,
 		    s->hostname,
 		    ss_to_text(&s->ss));
 	}
-	iobuf_fqueue(&s->obuf, "\n\tby %s (%s) with %sSMTP%s%s id %08x",
+	iobuf_fqueue(&s->tx->obuf, "\n\tby %s (%s) with %sSMTP%s%s id %08x",
 	    s->smtpname,
 	    SMTPD_NAME,
 	    s->flags & SF_EHLO ? "E" : "",
 	    s->flags & SF_SECURE ? "S" : "",
 	    s->flags & SF_AUTHENTICATED ? "A" : "",
-	    evpid_to_msgid(s->evp.id));
+	    s->tx->msgid);
 
 	if (s->flags & SF_SECURE) {
 		x = SSL_get_peer_certificate(s->io.ssl);
-		iobuf_fqueue(&s->obuf,
-		    "\n\tTLS version=%s cipher=%s bits=%d verify=%s",
+		iobuf_fqueue(&s->tx->obuf,
+		    " (%s:%s:%d:%s)",
 		    SSL_get_version(s->io.ssl),
 		    SSL_get_cipher_name(s->io.ssl),
 		    SSL_get_cipher_bits(s->io.ssl, NULL),
@@ -1211,27 +1214,23 @@ smtp_filter_fd(uint64_t id, int fd)
 			X509_free(x);
 
 		if (s->listener->flags & F_RECEIVEDAUTH) {
-			iobuf_fqueue(&s->obuf, " auth=%s", s->username[0] ? "yes" : "no");
+			iobuf_fqueue(&s->tx->obuf, " auth=%s", s->username[0] ? "yes" : "no");
 			if (s->username[0])
-				iobuf_fqueue(&s->obuf, " user=%s", s->username);
+				iobuf_fqueue(&s->tx->obuf, " user=%s", s->username);
 		}
 	}
 
-	if (s->rcptcount == 1) {
-		iobuf_fqueue(&s->obuf, "\n\tfor <%s@%s>",
-		    s->evp.rcpt.user,
-		    s->evp.rcpt.domain);
+	if (s->tx->rcptcount == 1) {
+		iobuf_fqueue(&s->tx->obuf, "\n\tfor <%s@%s>",
+		    s->tx->evp.rcpt.user,
+		    s->tx->evp.rcpt.domain);
 	}
 
-	iobuf_fqueue(&s->obuf, ";\n\t%s\n", time_to_text(time(NULL)));
+	iobuf_fqueue(&s->tx->obuf, ";\n\t%s\n", time_to_text(time(NULL)));
 
-	/*
-	 * XXX This is not exactly fair, since this is not really
-	 * user data.
-	 */
-	s->odatalen = iobuf_queued(&s->obuf);
+	s->tx->odatalen = iobuf_queued(&s->tx->obuf);
 
-	io_set_write(&s->oev);
+	io_set_write(&s->tx->oev);
 
 	smtp_enter_state(s, STATE_BODY);
 	smtp_reply(s, "354 Enter mail, end with \".\""
@@ -1256,7 +1255,7 @@ smtp_io(struct io *io, int evt)
 	switch (evt) {
 
 	case IO_TLSREADY:
-		log_info("smtp-in: Started TLS on session %016"PRIx64": %s",
+		log_info("%016"PRIx64" smtp event=starttls ciphers=\"%s\"",
 		    s->id, ssl_to_text(s->io.ssl));
 
 		s->flags |= SF_SECURE;
@@ -1268,8 +1267,9 @@ smtp_io(struct io *io, int evt)
 		}
 
 		if (s->listener->flags & F_TLS_VERIFY) {
-			log_info("smtp-in: Disconnecting session %016" PRIx64
-			    ": client did not present certificate", s->id);
+			log_info("%016"PRIx64" smtp "
+			    "event=closed reason=no-client-cert",
+			    s->id);
 			smtp_free(s, "client did not present certificate");
 			return;
 		}
@@ -1279,10 +1279,10 @@ smtp_io(struct io *io, int evt)
 	case IO_TLSVERIFIED:
 		x = SSL_get_peer_certificate(s->io.ssl);
 		if (x) {
-			log_info("smtp-in: Client certificate verification %s "
-			    "on session %016"PRIx64,
-			    (s->flags & SF_VERIFIED) ? "succeeded" : "failed",
-			    s->id);
+			log_info("%016"PRIx64" smtp "
+			    "event=client-cert-check result=\"%s\"",
+			    s->id,
+			    (s->flags & SF_VERIFIED) ? "success" : "failure");
 			X509_free(x);
 		}
 
@@ -1337,14 +1337,16 @@ smtp_io(struct io *io, int evt)
 		if (s->state == STATE_BODY) {
 			log_trace(TRACE_SMTP, "<<< [EOM]");
 
-			rfc2822_parser_flush(&s->rfc2822_parser);
+			rfc2822_parser_flush(&s->tx->rfc2822_parser);
 
 			iobuf_normalize(&s->iobuf);
 			io_set_write(io);
 
-			s->dataeom = 1;
-			if (iobuf_queued(&s->obuf) == 0)
+			s->tx->dataeom = 1;
+			if (iobuf_queued(&s->tx->obuf) == 0)
 				smtp_data_io_done(s);
+			else
+				io_reload(&s->tx->oev);
 			return;
 		}
 
@@ -1357,7 +1359,8 @@ smtp_io(struct io *io, int evt)
 
 	case IO_LOWAT:
 		if (s->state == STATE_QUIT) {
-			log_info("smtp-in: Closing session %016" PRIx64, s->id);
+			log_info("%016"PRIx64" smtp event=closed reason=quit",
+			    s->id);
 			smtp_free(s, "done");
 			break;
 		}
@@ -1386,20 +1389,20 @@ smtp_io(struct io *io, int evt)
 		break;
 
 	case IO_TIMEOUT:
-		log_info("smtp-in: Disconnecting session %016"PRIx64": "
-		    "session timeout", s->id);
+		log_info("%016"PRIx64" smtp event=closed reason=timeout",
+		    s->id);
 		smtp_free(s, "timeout");
 		break;
 
 	case IO_DISCONNECTED:
-		log_info("smtp-in: Received disconnect from session %016"PRIx64,
+		log_info("%016"PRIx64" smtp event=closed reason=disconnect",
 		    s->id);
 		smtp_free(s, "disconnected");
 		break;
 
 	case IO_ERROR:
-		log_info("smtp-in: Disconnecting session %016"PRIx64": "
-		    "IO error: %s", s->id, io->error);
+		log_info("%016"PRIx64" smtp event=closed reason=\"io-error: %s\"",
+		    s->id, io->error);
 		smtp_free(s, "IO error");
 		break;
 
@@ -1408,6 +1411,75 @@ smtp_io(struct io *io, int evt)
 	}
 }
 
+static int
+smtp_tx(struct smtp_session *s)
+{
+	struct smtp_tx *tx;
+
+	tx = calloc(1, sizeof(*tx));
+	if (tx == NULL)
+		return 0;
+
+	TAILQ_INIT(&tx->rcpts);
+	io_init(&tx->oev, -1, s, NULL, NULL); /* initialise 'sock', but not to 0 */
+
+	s->tx = tx;
+	tx->session = s;
+
+	/* setup the envelope */
+	s->tx->evp.ss = s->ss;
+	(void)strlcpy(s->tx->evp.tag, s->listener->tag, sizeof(s->tx->evp.tag));
+	(void)strlcpy(s->tx->evp.smtpname, s->smtpname, sizeof(s->tx->evp.smtpname));
+	(void)strlcpy(s->tx->evp.hostname, s->hostname, sizeof s->tx->evp.hostname);
+	(void)strlcpy(s->tx->evp.helo, s->helo, sizeof s->tx->evp.helo);
+
+	if (s->flags & SF_BOUNCE)
+		s->tx->evp.flags |= EF_BOUNCE;
+	if (s->flags & SF_AUTHENTICATED)
+		s->tx->evp.flags |= EF_AUTHENTICATED;
+
+	/* Setup parser and callbacks */
+	rfc2822_parser_init(&tx->rfc2822_parser);
+	rfc2822_header_default_callback(&tx->rfc2822_parser,
+	    header_default_callback, s);
+	rfc2822_header_callback(&tx->rfc2822_parser, "bcc",
+	    header_bcc_callback, s);
+	rfc2822_header_callback(&tx->rfc2822_parser, "from",
+	    header_domain_append_callback, s);
+	rfc2822_header_callback(&tx->rfc2822_parser, "to",
+	    header_domain_append_callback, s);
+	rfc2822_header_callback(&tx->rfc2822_parser, "cc",
+	    header_domain_append_callback, s);
+	rfc2822_body_callback(&tx->rfc2822_parser,
+	    dataline_callback, s);
+
+	if (s->listener->local || s->listener->port == 587) {
+		rfc2822_missing_header_callback(&tx->rfc2822_parser, "date",
+		    header_missing_callback, s);
+		rfc2822_missing_header_callback(&tx->rfc2822_parser, "message-id",
+		    header_missing_callback, s);
+	}
+
+	return 1;
+}
+
+static void
+smtp_tx_free(struct smtp_tx *tx)
+{
+	struct smtp_rcpt *rcpt;
+
+	rfc2822_parser_release(&tx->rfc2822_parser);
+
+	while ((rcpt = TAILQ_FIRST(&tx->rcpts))) {
+		TAILQ_REMOVE(&tx->rcpts, rcpt, entry);
+		free(rcpt);
+	}
+
+	tx->session->tx = NULL;
+
+	free(tx);
+}
+
 static void
 smtp_data_io(struct io *io, int evt)
 {
@@ -1421,9 +1493,9 @@ smtp_data_io(struct io *io, int evt)
 	case IO_DISCONNECTED:
 	case IO_ERROR:
 		log_debug("debug: smtp: %p: io error on mfa", s);
-		io_clear(&s->oev);
-		iobuf_clear(&s->obuf);
-		s->msgflags |= MF_ERROR_IO;
+		io_clear(&s->tx->oev);
+		iobuf_clear(&s->tx->obuf);
+		s->tx->msgflags |= MF_ERROR_IO;
 		if (s->io.flags & IO_PAUSE_IN) {
 			log_debug("debug: smtp: %p: resuming session after mfa error", s);
 			io_resume(&s->io, IO_PAUSE_IN);
@@ -1431,7 +1503,7 @@ smtp_data_io(struct io *io, int evt)
 		break;
 
 	case IO_LOWAT:
-		if (s->dataeom && iobuf_queued(&s->obuf) == 0) {
+		if (s->tx->dataeom && iobuf_queued(&s->tx->obuf) == 0) {
 			smtp_data_io_done(s);
 		} else if (s->io.flags & IO_PAUSE_IN) {
 			log_debug("debug: smtp: %p: filter congestion over: resuming session", s);
@@ -1447,35 +1519,36 @@ smtp_data_io(struct io *io, int evt)
 static void
 smtp_data_io_done(struct smtp_session *s)
 {
-	log_debug("debug: smtp: %p: data io done (%zu bytes)", s, s->odatalen);
-	io_clear(&s->oev);
-	iobuf_clear(&s->obuf);
+	log_debug("debug: smtp: %p: data io done (%zu bytes)", s, s->tx->odatalen);
+	io_clear(&s->tx->oev);
+	iobuf_clear(&s->tx->obuf);
 
-	if (s->msgflags & MF_ERROR) {
+	if (s->tx->msgflags & MF_ERROR) {
 
 		tree_pop(&wait_filter_data, s->id);
 
-		smtp_filter_rollback(s);
+		smtp_filter_tx_rollback(s);
 		smtp_queue_rollback(s);
 
-		if (s->msgflags & MF_ERROR_SIZE)
+		if (s->tx->msgflags & MF_ERROR_SIZE)
 			smtp_reply(s, "554 Message too big");
-		else if (s->msgflags & MF_ERROR_LOOP)
+		else if (s->tx->msgflags & MF_ERROR_LOOP)
 			smtp_reply(s, "500 %s %s: Loop detected",
 				esc_code(ESC_STATUS_PERMFAIL, ESC_ROUTING_LOOP_DETECTED),
 				esc_description(ESC_ROUTING_LOOP_DETECTED));
-                else if (s->msgflags & MF_ERROR_RESOURCES)
+                else if (s->tx->msgflags & MF_ERROR_RESOURCES)
                         smtp_reply(s, "421 %s: Temporary Error",
                             esc_code(ESC_STATUS_TEMPFAIL, ESC_OTHER_MAIL_SYSTEM_STATUS));
-                else if (s->msgflags & MF_ERROR_MALFORMED)
+                else if (s->tx->msgflags & MF_ERROR_MALFORMED)
                         smtp_reply(s, "550 %s %s: Message is not RFC 2822 compliant",
                             esc_code(ESC_STATUS_PERMFAIL,
 				ESC_DELIVERY_NOT_AUTHORIZED_MESSAGE_REFUSED),
                             esc_description(ESC_DELIVERY_NOT_AUTHORIZED_MESSAGE_REFUSED));
-		else if (s->msgflags)
+		else if (s->tx->msgflags)
 			smtp_reply(s, "421 Internal server error");
-		smtp_message_reset(s, 0);
+		smtp_tx_free(s->tx);
 		smtp_enter_state(s, STATE_HELO);
+		s->phase = PHASE_SETUP;
 		io_reload(&s->io);
 	}
 	else {
@@ -1561,8 +1634,6 @@ smtp_command(struct smtp_session *s, char *line)
 			s->flags |= SF_8BITMIME;
 		}
 
-		smtp_message_reset(s, 1);
-
 		smtp_filter_helo(s);
 		break;
 	/*
@@ -1681,17 +1752,26 @@ smtp_command(struct smtp_session *s, char *line)
 			break;
 		}
 
-		smtp_message_reset(s, 1);
+		if (!smtp_tx(s)) {
+			smtp_reply(s, "421 %s: Temporary Error",
+			    esc_code(ESC_STATUS_TEMPFAIL, ESC_OTHER_MAIL_SYSTEM_STATUS));
+			smtp_enter_state(s, STATE_QUIT);
+			break;
+		}
 
-		if (smtp_mailaddr(&s->evp.sender, args, 1, &args,
+		if (smtp_mailaddr(&s->tx->evp.sender, args, 1, &args,
 			s->smtpname) == 0) {
+			smtp_tx_free(s->tx);
 			smtp_reply(s, "553 %s: Sender address syntax error",
 			    esc_code(ESC_STATUS_PERMFAIL, ESC_OTHER_ADDRESS_STATUS));
 			break;
 		}
-		if (args && smtp_parse_mail_args(s, args) == -1)
+		if (args && smtp_parse_mail_args(s, args) == -1) {
+			smtp_tx_free(s->tx);
 			break;
+		}
 
+		smtp_filter_tx_begin(s);
 		smtp_filter_mail(s);
 		break;
 	/*
@@ -1705,14 +1785,14 @@ smtp_command(struct smtp_session *s, char *line)
 			break;
 		}
 
-		if (s->rcptcount >= env->sc_session_max_rcpt) {
+		if (s->tx->rcptcount >= env->sc_session_max_rcpt) {
 			smtp_reply(s, "451 %s %s: Too many recipients",
 			    esc_code(ESC_STATUS_TEMPFAIL, ESC_TOO_MANY_RECIPIENTS),
 			    esc_description(ESC_TOO_MANY_RECIPIENTS));
 			break;
 		}
 
-		if (smtp_mailaddr(&s->evp.rcpt, args, 0, &args,
+		if (smtp_mailaddr(&s->tx->evp.rcpt, args, 0, &args,
 		    s->smtpname) == 0) {
 			smtp_reply(s,
 			    "501 %s: Recipient address syntax error",
@@ -1733,13 +1813,16 @@ smtp_command(struct smtp_session *s, char *line)
 			break;
 		}
 
+		if (s->tx) {
+			smtp_filter_tx_rollback(s);
+			if (s->tx->msgid)
+				smtp_queue_rollback(s);
+			smtp_tx_free(s->tx);
+		}
+
 		smtp_filter_rset(s);
 
-		if (s->evp.id)
-			smtp_queue_rollback(s);
-
 		s->phase = PHASE_SETUP;
-		smtp_message_reset(s, 0);
 		smtp_reply(s, "250 %s: Reset state",
 		    esc_code(ESC_STATUS_OK, ESC_OTHER_STATUS));
 		break;
@@ -1751,15 +1834,13 @@ smtp_command(struct smtp_session *s, char *line)
 			    esc_description(ESC_INVALID_COMMAND));
 			break;
 		}
-		if (s->rcptcount == 0) {
+		if (s->tx->rcptcount == 0) {
 			smtp_reply(s, "503 %s %s: No recipient specified",
 			    esc_code(ESC_STATUS_PERMFAIL, ESC_INVALID_COMMAND_ARGUMENTS),
 			    esc_description(ESC_INVALID_COMMAND_ARGUMENTS));
 			break;
 		}
 
-		rfc2822_parser_reset(&s->rfc2822_parser);
-
 		smtp_filter_data(s);
 		break;
 	/*
@@ -1940,10 +2021,10 @@ smtp_parse_rcpt_args(struct smtp_session *s, char *args)
 				if ((flag = dsn_notify_str_to_uint8(p)) == 0)
 					continue;
 
-				s->evp.dsn_notify |= flag;
+				s->tx->evp.dsn_notify |= flag;
 			}
-			if (s->evp.dsn_notify & DSN_NEVER &&
-			    s->evp.dsn_notify & (DSN_SUCCESS | DSN_FAILURE |
+			if (s->tx->evp.dsn_notify & DSN_NEVER &&
+			    s->tx->evp.dsn_notify & (DSN_SUCCESS | DSN_FAILURE |
 			    DSN_DELAY)) {
 				smtp_reply(s,
 				    "553 NOTIFY option NEVER cannot be \
@@ -1952,7 +2033,7 @@ smtp_parse_rcpt_args(struct smtp_session *s, char *args)
 			}
 		} else if (ADVERTISE_EXT_DSN(s) && strncasecmp(b, "ORCPT=", 6) == 0) {
 			b += 6;
-			if (!text_to_mailaddr(&s->evp.dsn_orcpt, b)) {
+			if (!text_to_mailaddr(&s->tx->evp.dsn_orcpt, b)) {
 				smtp_reply(s, "553 ORCPT address syntax error");
 				return (-1);
 			}
@@ -1986,13 +2067,13 @@ smtp_parse_mail_args(struct smtp_session *s, char *args)
 		else if (ADVERTISE_EXT_DSN(s) && strncasecmp(b, "RET=", 4) == 0) {
 			b += 4;
 			if (strcasecmp(b, "HDRS") == 0)
-				s->evp.dsn_ret = DSN_RETHDRS;
+				s->tx->evp.dsn_ret = DSN_RETHDRS;
 			else if (strcasecmp(b, "FULL") == 0)
-				s->evp.dsn_ret = DSN_RETFULL;
+				s->tx->evp.dsn_ret = DSN_RETFULL;
 		} else if (ADVERTISE_EXT_DSN(s) && strncasecmp(b, "ENVID=", 6) == 0) {
 			b += 6;
-			if (strlcpy(s->evp.dsn_envid, b, sizeof(s->evp.dsn_envid))
-			    >= sizeof(s->evp.dsn_envid)) {
+			if (strlcpy(s->tx->evp.dsn_envid, b, sizeof(s->tx->evp.dsn_envid))
+			    >= sizeof(s->tx->evp.dsn_envid)) {
 				smtp_reply(s, "503 %s %s: option too large, truncated: %s",
 				    esc_code(ESC_STATUS_PERMFAIL, ESC_INVALID_COMMAND_ARGUMENTS),
 				    esc_description(ESC_INVALID_COMMAND_ARGUMENTS), b);
@@ -2043,8 +2124,8 @@ smtp_connected(struct smtp_session *s)
 
 	smtp_enter_state(s, STATE_CONNECTED);
 
-	log_info("smtp-in: New session %016"PRIx64" from host %s [%s]",
-	    s->id, s->hostname, ss_to_text(&s->ss));
+	log_info("%016"PRIx64" smtp event=connected address=%s host=%s",
+	    s->id, ss_to_text(&s->ss), s->hostname);
 
 	sl = sizeof(ss);
 	if (getsockname(s->io.sock, (struct sockaddr*)&ss, &sl) == -1) {
@@ -2076,21 +2157,22 @@ smtp_enter_state(struct smtp_session *s, int newstate)
 static void
 smtp_message_end(struct smtp_session *s)
 {
-	log_debug("debug: %p: end of message, msgflags=0x%04x", s, s->msgflags);
+	log_debug("debug: %p: end of message, msgflags=0x%04x", s, s->tx->msgflags);
 
 	tree_xpop(&wait_filter_data, s->id);
 
 	s->phase = PHASE_SETUP;
 
-	if (s->msgflags & MF_ERROR) {
+	if (s->tx->msgflags & MF_ERROR) {
+		smtp_filter_tx_rollback(s);
 		smtp_queue_rollback(s);
-		if (s->msgflags & MF_ERROR_SIZE)
+		if (s->tx->msgflags & MF_ERROR_SIZE)
 			smtp_reply(s, "554 %s %s: Transaction failed, message too big",
 			    esc_code(ESC_STATUS_PERMFAIL, ESC_MESSAGE_TOO_BIG_FOR_SYSTEM),
 			    esc_description(ESC_MESSAGE_TOO_BIG_FOR_SYSTEM));
 		else
-			smtp_reply(s, "%d Message rejected", s->msgcode);
-		smtp_message_reset(s, 0);
+			smtp_reply(s, "%d Message rejected", s->tx->msgcode);
+		smtp_tx_free(s->tx);
 		smtp_enter_state(s, STATE_HELO);
 		return;
 	}
@@ -2098,59 +2180,25 @@ smtp_message_end(struct smtp_session *s)
 	smtp_queue_commit(s);
 }
 
-static void
-smtp_message_reset(struct smtp_session *s, int prepare)
-{
-	struct smtp_rcpt	*rcpt;
-
-	while ((rcpt = TAILQ_FIRST(&s->rcpts))) {
-		TAILQ_REMOVE(&s->rcpts, rcpt, entry);
-		free(rcpt);
-	}
-
-	memset(&s->evp, 0, sizeof s->evp);
-	s->msgflags = 0;
-	s->destcount = 0;
-	s->rcptcount = 0;
-	s->datain = 0;
-	s->odatalen = 0;
-	s->dataeom = 0;
-	s->rcvcount = 0;
-	s->hdrdone = 0;
-
-	if (prepare) {
-		s->evp.ss = s->ss;
-		(void)strlcpy(s->evp.tag, s->listener->tag, sizeof(s->evp.tag));
-		(void)strlcpy(s->evp.smtpname, s->smtpname, sizeof(s->evp.smtpname));
-		(void)strlcpy(s->evp.hostname, s->hostname, sizeof s->evp.hostname);
-		(void)strlcpy(s->evp.helo, s->helo, sizeof s->evp.helo);
-
-		if (s->flags & SF_BOUNCE)
-			s->evp.flags |= EF_BOUNCE;
-		if (s->flags & SF_AUTHENTICATED)
-			s->evp.flags |= EF_AUTHENTICATED;
-	}
-}
-
 static int
 smtp_message_printf(struct smtp_session *s, const char *fmt, ...)
 {
 	va_list	ap;
 	int	len;
 
-	if (s->msgflags & MF_ERROR)
+	if (s->tx->msgflags & MF_ERROR)
 		return -1;
 
 	va_start(ap, fmt);
-	len = iobuf_vfqueue(&s->obuf, fmt, ap);
+	len = iobuf_vfqueue(&s->tx->obuf, fmt, ap);
 	va_end(ap);
 
 	if (len < 0) {
 		log_warn("smtp-in: session %016"PRIx64": vfprintf", s->id);
-		s->msgflags |= MF_ERROR_IO;
+		s->tx->msgflags |= MF_ERROR_IO;
 	}
 	else
-		s->odatalen += len;
+		s->tx->odatalen += len;
 
 	return len;
 }
@@ -2178,8 +2226,9 @@ smtp_reply(struct smtp_session *s, char *fmt, ...)
 	case '5':
 	case '4':
 		if (s->flags & SF_BADINPUT) {
-			log_info("smtp-in: Bad input on session %016"PRIx64
-			    ": %.*s", s->id, n, buf);
+			log_info("%016"PRIx64" smtp "
+			    "event=bad-input result=\"%.*s\"",
+			    s->id, n, buf);
 		}
 		else if (s->state == STATE_AUTH_INIT) {
 			log_info("smtp-in: Failed command on session %016"PRIx64
@@ -2195,8 +2244,9 @@ smtp_reply(struct smtp_session *s, char *fmt, ...)
 		}
 		else {
 			strnvis(tmp, s->cmd, sizeof tmp, VIS_SAFE | VIS_CSTYLE);
-			log_info("smtp-in: Failed command on session %016"PRIx64
-			    ": \"%s\" => %.*s", s->id, tmp, n, buf);
+			log_info("%016"PRIx64" smtp "
+			    "event=failed-command command=\"%s\" result=\"%.*s\"",
+			    s->id, tmp, n, buf);
 		}
 		break;
 	}
@@ -2205,16 +2255,18 @@ smtp_reply(struct smtp_session *s, char *fmt, ...)
 static void
 smtp_free(struct smtp_session *s, const char * reason)
 {
-	struct smtp_rcpt	*rcpt;
-
 	log_debug("debug: smtp: %p: deleting session: %s", s, reason);
 
 	tree_pop(&wait_filter_data, s->id);
 
-	if (s->evp.id) {
-		smtp_queue_rollback(s);
-		io_clear(&s->oev);
-		iobuf_clear(&s->obuf);
+	if (s->tx) {
+		if (s->tx->msgid) {
+			smtp_queue_rollback(s);
+			io_clear(&s->tx->oev);
+			iobuf_clear(&s->tx->obuf);
+		}
+		smtp_filter_tx_rollback(s);
+		smtp_tx_free(s->tx);
 	}
 
 	if (s->flags & SF_FILTERCONN)
@@ -2225,13 +2277,6 @@ smtp_free(struct smtp_session *s, const char * reason)
 	if (s->flags & SF_SECURE && s->listener->flags & F_STARTTLS)
 		stat_decrement("smtp.tls", 1);
 
-	while ((rcpt = TAILQ_FIRST(&s->rcpts))) {
-		TAILQ_REMOVE(&s->rcpts, rcpt, entry);
-		free(rcpt);
-	}
-
-	rfc2822_parser_release(&s->rfc2822_parser);
-
 	io_clear(&s->io);
 	iobuf_clear(&s->iobuf);
 	free(s);
@@ -2450,7 +2495,7 @@ smtp_queue_open_message(struct smtp_session *s)
 {
 	m_create(p_queue, IMSG_SMTP_MESSAGE_OPEN, 0, 0, -1);
 	m_add_id(p_queue, s->id);
-	m_add_msgid(p_queue, evpid_to_msgid(s->evp.id));
+	m_add_msgid(p_queue, s->tx->msgid);
 	m_close(p_queue);
 	tree_xset(&wait_queue_fd, s->id, s);
 }
@@ -2460,7 +2505,7 @@ smtp_queue_commit(struct smtp_session *s)
 {
 	m_create(p_queue, IMSG_SMTP_MESSAGE_COMMIT, 0, 0, -1);
 	m_add_id(p_queue, s->id);
-	m_add_msgid(p_queue, evpid_to_msgid(s->evp.id));
+	m_add_msgid(p_queue, s->tx->msgid);
 	m_close(p_queue);
 	tree_xset(&wait_queue_commit, s->id, s);
 }
@@ -2469,7 +2514,7 @@ static void
 smtp_queue_rollback(struct smtp_session *s)
 {
 	m_create(p_queue, IMSG_SMTP_MESSAGE_ROLLBACK, 0, 0, -1);
-	m_add_msgid(p_queue, evpid_to_msgid(s->evp.id));
+	m_add_msgid(p_queue, s->tx->msgid);
 	m_close(p_queue);
 }
 
@@ -2480,15 +2525,24 @@ smtp_filter_rset(struct smtp_session *s)
 }
 
 static void
-smtp_filter_commit(struct smtp_session *s)
+smtp_filter_tx_begin(struct smtp_session *s)
 {
-	filter_event(s->id, EVENT_COMMIT);
+	s->flags |= SF_FILTERTX;
+	filter_event(s->id, EVENT_TX_BEGIN);
 }
 
 static void
-smtp_filter_rollback(struct smtp_session *s)
+smtp_filter_tx_commit(struct smtp_session *s)
 {
-	filter_event(s->id, EVENT_ROLLBACK);
+	s->flags &= ~SF_FILTERTX;
+	filter_event(s->id, EVENT_TX_COMMIT);
+}
+
+static void
+smtp_filter_tx_rollback(struct smtp_session *s)
+{
+	s->flags &= ~SF_FILTERTX;
+	filter_event(s->id, EVENT_TX_ROLLBACK);
 }
 
 static void
@@ -2513,7 +2567,7 @@ static void
 smtp_filter_eom(struct smtp_session *s)
 {
 	tree_xset(&wait_filter, s->id, s);
-	filter_eom(s->id, QUERY_EOM, s->odatalen);
+	filter_eom(s->id, QUERY_EOM, s->tx->odatalen);
 }
 
 static void
@@ -2527,14 +2581,14 @@ static void
 smtp_filter_mail(struct smtp_session *s)
 {
 	tree_xset(&wait_filter, s->id, s);
-	filter_mailaddr(s->id, QUERY_MAIL, &s->evp.sender);
+	filter_mailaddr(s->id, QUERY_MAIL, &s->tx->evp.sender);
 }
 
 static void
 smtp_filter_rcpt(struct smtp_session *s)
 {
 	tree_xset(&wait_filter, s->id, s);
-	filter_mailaddr(s->id, QUERY_RCPT, &s->evp.rcpt);
+	filter_mailaddr(s->id, QUERY_RCPT, &s->tx->evp.rcpt);
 }
 
 static void
@@ -2552,7 +2606,7 @@ smtp_filter_dataline(struct smtp_session *s, const char *line)
 	log_trace(TRACE_SMTP, "<<< [MSG] %s", line);
 
 	/* ignore data line if an error flag is set */
-	if (s->msgflags & MF_ERROR)
+	if (s->tx->msgflags & MF_ERROR)
 		return;
 
 	/* escape lines starting with a '.' */
@@ -2560,54 +2614,54 @@ smtp_filter_dataline(struct smtp_session *s, const char *line)
 		line += 1;
 
 	/* account for newline */
-	s->datain += strlen(line) + 1;
-	if (s->datain > env->sc_maxsize) {
-		s->msgflags |= MF_ERROR_SIZE;
+	s->tx->datain += strlen(line) + 1;
+	if (s->tx->datain > env->sc_maxsize) {
+		s->tx->msgflags |= MF_ERROR_SIZE;
 		return;
 	}
 
-	if (!s->hdrdone) {
+	if (!s->tx->hdrdone) {
 
 		/* folded header that must be skipped */
-		if (isspace((unsigned char)line[0]) && s->skiphdr)
+		if (isspace((unsigned char)line[0]) && s->tx->skiphdr)
 			return;
-		s->skiphdr = 0;
+		s->tx->skiphdr = 0;
 
 		/* BCC should be stripped from headers */
 		if (strncasecmp("bcc:", line, 4) == 0) {
-			s->skiphdr = 1;
+			s->tx->skiphdr = 1;
 			return;
 		}
 
 		/* check for loop */
 		if (strncasecmp("Received: ", line, 10) == 0)
-			s->rcvcount++;
-		if (s->rcvcount == MAX_HOPS_COUNT) {
-			s->msgflags |= MF_ERROR_LOOP;
-			log_warn("warn: loop detected");
+			s->tx->rcvcount++;
+		if (s->tx->rcvcount == MAX_HOPS_COUNT) {
+			s->tx->msgflags |= MF_ERROR_LOOP;
+			log_warnx("warn: loop detected");
 			return;
 		}
 
 		if (line[0] == '\0')
-			s->hdrdone = 1;
+			s->tx->hdrdone = 1;
 	}
 
-	ret = rfc2822_parser_feed(&s->rfc2822_parser, line);
+	ret = rfc2822_parser_feed(&s->tx->rfc2822_parser, line);
 	if (ret == -1) {
-		s->msgflags |= MF_ERROR_RESOURCES;
+		s->tx->msgflags |= MF_ERROR_RESOURCES;
 		return;
 	}
 
 	if (ret == 0) {
-		s->msgflags |= MF_ERROR_MALFORMED;
+		s->tx->msgflags |= MF_ERROR_MALFORMED;
 		return;
 	}
 
-	if (iobuf_queued(&s->obuf) > DATA_HIWAT && !(s->io.flags & IO_PAUSE_IN)) {
+	if (iobuf_queued(&s->tx->obuf) > DATA_HIWAT && !(s->io.flags & IO_PAUSE_IN)) {
 		log_debug("debug: smtp: %p: filter congestion over: pausing session", s);
 		io_pause(&s->io, IO_PAUSE_IN);
 	}
-	io_reload(&s->oev);
+	io_reload(&s->tx->oev);
 }
 
 #define CASE(x) case x : return #x
