@@ -1,4 +1,4 @@
-/*	$OpenBSD: smtpd.c,v 1.274 2016/02/05 19:15:15 jung Exp $	*/
+/*	$OpenBSD: smtpd.c,v 1.279 2016/06/20 20:26:04 gilles Exp $	*/
 
 /*
  * Copyright (c) 2008 Gilles Chehade <gilles@poolp.org>
@@ -61,6 +61,7 @@
 #ifdef HAVE_PATHS_H
 #include <paths.h>
 #endif
+#include <poll.h>
 #include <pwd.h>
 #include <signal.h>
 #ifdef HAVE_SHADOW_H
@@ -85,8 +86,11 @@
 
 extern char *__progname;
 
+#define SMTPD_MAXARG 32
+
 static void parent_imsg(struct mproc *, struct imsg *);
 static void usage(void);
+static int smtpd(void);
 static void parent_shutdown(int);
 static void parent_send_config(int, short, void *);
 static void parent_send_config_lka(void);
@@ -95,8 +99,14 @@ static void parent_send_config_ca(void);
 static void parent_sig_handler(int, short, void *);
 static void forkmda(struct mproc *, uint64_t, struct deliver *);
 static int parent_forward_open(char *, char *, uid_t, gid_t);
-static void fork_peers(void);
 static struct child *child_add(pid_t, int, const char *);
+static struct mproc *start_child(int, char **, char *);
+static struct mproc *setup_peer(enum smtp_proc_type, pid_t, int);
+static void setup_peers(struct mproc *, struct mproc *);
+static void setup_done(struct mproc *);
+static void setup_proc(void);
+static struct mproc *setup_peer(enum smtp_proc_type, pid_t, int);
+static int imsg_wait(struct imsgbuf *, struct imsg *, int);
 
 static void	offline_scan(int, short, void *);
 static int	offline_add(char *);
@@ -186,6 +196,9 @@ parent_imsg(struct mproc *p, struct imsg *imsg)
 	void			*i;
 	int			 fd, n, v, ret;
 
+	if (imsg == NULL)
+		exit(1);
+	
 	if (p->proc == PROC_LKA) {
 		switch (imsg->hdr.type) {
 		case IMSG_LKA_OPEN_FORWARD:
@@ -476,12 +489,10 @@ main(int argc, char *argv[])
 	int		 c, i;
 	int		 opts, flags;
 	const char	*conffile = CONF_FILE;
-	struct smtpd	 smtpd;
-	struct event	 ev_sigint;
-	struct event	 ev_sigterm;
-	struct event	 ev_sigchld;
-	struct event	 ev_sighup;
-	struct timeval	 tv;
+	int		 save_argc = argc;
+	char		**save_argv = argv;
+	char		*rexec = NULL;
+	struct smtpd	 conf;
 
 	__progname = ssh_get_progname(argv[0]);
 
@@ -498,7 +509,11 @@ main(int argc, char *argv[])
 	argv = saved_argv;
 #endif
 
-	env = &smtpd;
+	/* this is to work around GNU getopt + portable setproctitle() fuckery */
+	save_argc = saved_argc;
+	save_argv = saved_argv;
+
+	env = &conf;
 
 	flags = 0;
 	opts = 0;
@@ -509,7 +524,7 @@ main(int argc, char *argv[])
 
 	TAILQ_INIT(&offline_q);
 
-	while ((c = getopt(argc, argv, "B:dD:hnP:f:FT:v")) != -1) {
+	while ((c = getopt(argc, argv, "B:dD:hnP:f:FT:vx:")) != -1) {
 		switch (c) {
 		case 'B':
 			if (strstr(optarg, "queue=") == optarg)
@@ -608,6 +623,9 @@ main(int argc, char *argv[])
 		case 'v':
 			verbose |=  TRACE_DEBUG;
 			break;
+		case 'x':
+			rexec = optarg;
+			break;
 		default:
 			usage();
 		}
@@ -621,7 +639,7 @@ main(int argc, char *argv[])
 
 	ssl_init();
 
-	if (parse_config(&smtpd, conffile, opts))
+	if (parse_config(&conf, conffile, opts))
 		exit(1);
 
 	seed_rng();
@@ -651,66 +669,12 @@ main(int argc, char *argv[])
 	if (geteuid())
 		errx(1, "need root privileges");
 
-	/* the control socket ensures that only one smtpd instance is running */
-	control_socket = control_create_socket();
-
-	if (!queue_init(backend_queue, 1))
-		errx(1, "could not initialize queue backend");
-
-	env->sc_stat = stat_backend_lookup(backend_stat);
-	if (env->sc_stat == NULL)
-		errx(1, "could not find stat backend \"%s\"", backend_stat);
-
-	if (env->sc_queue_flags & QUEUE_ENCRYPTION) {
-		if (env->sc_queue_key == NULL) {
-			char	*password;
-
-			password = getpass("queue key: ");
-			if (password == NULL)
-				err(1, "getpass");
-
-			env->sc_queue_key = strdup(password);
-			explicit_bzero(password, strlen(password));
-			if (env->sc_queue_key == NULL)
-				err(1, "strdup");
-		}
-		else {
-			char   *buf = NULL;
-			size_t	sz = 0;
-			ssize_t	len;
-
-			if (strcasecmp(env->sc_queue_key, "stdin") == 0) {
-				if ((len = getline(&buf, &sz, stdin)) == -1)
-					err(1, "getline");
-				if (buf[len - 1] == '\n')
-					buf[len - 1] = '\0';
-				env->sc_queue_key = buf;
-			}
-		}
-	}
-
-	if (env->sc_queue_flags & QUEUE_COMPRESSION)
-		env->sc_comp = compress_backend_lookup("gzip");
-
 	log_init(foreground_log);
 	log_verbose(verbose);
 
 	load_pki_tree();
 	load_pki_keys();
 
-	log_info("info: %s %s starting", SMTPD_NAME, SMTPD_VERSION);
-
-	if (!foreground)
-		if (daemon(0, 0) == -1)
-			err(1, "failed to daemonize");
-
-	for (i = 0; i < MAX_BOUNCE_WARN; i++) {
-		if (env->sc_bounce_warn[i] == 0)
-			break;
-		log_debug("debug: bounce warning after %s",
-		    duration_to_text(env->sc_bounce_warn[i]));
-	}
-
 	log_debug("debug: using \"%s\" queue backend", backend_queue);
 	log_debug("debug: using \"%s\" scheduler backend", backend_scheduler);
 	log_debug("debug: using \"%s\" stat backend", backend_stat);
@@ -719,9 +683,403 @@ main(int argc, char *argv[])
 		errx(1, "machine does not have a hostname set");
 	env->sc_uptime = time(NULL);
 
-	fork_peers();
+	if (rexec == NULL) {
+		smtpd_process = PROC_PARENT;
+
+		if (env->sc_queue_flags & QUEUE_ENCRYPTION) {
+			if (env->sc_queue_key == NULL) {
+				char	*password;
+
+				password = getpass("queue key: ");
+				if (password == NULL)
+					err(1, "getpass");
+
+				env->sc_queue_key = strdup(password);
+				explicit_bzero(password, strlen(password));
+				if (env->sc_queue_key == NULL)
+					err(1, "strdup");
+			}
+			else {
+				char   *buf = NULL;
+				size_t	sz = 0;
+				ssize_t	len;
+
+				if (strcasecmp(env->sc_queue_key, "stdin") == 0) {
+					if ((len = getline(&buf, &sz, stdin)) == -1)
+						err(1, "getline");
+					if (buf[len - 1] == '\n')
+						buf[len - 1] = '\0';
+					env->sc_queue_key = buf;
+				}
+			}
+		}
+
+		log_info("info: %s %s starting", SMTPD_NAME, SMTPD_VERSION);
+
+		if (!foreground)
+			if (daemon(0, 0) == -1)
+				err(1, "failed to daemonize");
+
+		/* setup all processes */
+
+		p_ca = start_child(save_argc, save_argv, "ca");
+		p_ca->proc = PROC_CA;
+
+		p_control = start_child(save_argc, save_argv, "control");
+		p_control->proc = PROC_CONTROL;
+
+		p_lka = start_child(save_argc, save_argv, "lka");
+		p_lka->proc = PROC_LKA;
+
+		p_pony = start_child(save_argc, save_argv, "pony");
+		p_pony->proc = PROC_PONY;
+
+		p_queue = start_child(save_argc, save_argv, "queue");
+		p_queue->proc = PROC_QUEUE;
+
+		p_scheduler = start_child(save_argc, save_argv, "scheduler");
+		p_scheduler->proc = PROC_SCHEDULER;
+
+		setup_peers(p_control, p_ca);
+		setup_peers(p_control, p_lka);
+		setup_peers(p_control, p_pony);
+		setup_peers(p_control, p_queue);
+		setup_peers(p_control, p_scheduler);
+		setup_peers(p_pony, p_ca);
+		setup_peers(p_pony, p_lka);
+		setup_peers(p_pony, p_queue);
+		setup_peers(p_queue, p_lka);
+		setup_peers(p_queue, p_scheduler);
+
+		if (env->sc_queue_key) {
+			if (imsg_compose(&p_queue->imsgbuf, IMSG_SETUP_KEY, 0,
+			    0, -1, env->sc_queue_key, strlen(env->sc_queue_key)
+			    + 1) == -1)
+				fatal("imsg_compose");
+			if (imsg_flush(&p_queue->imsgbuf) == -1)
+				fatal("imsg_flush");
+		}
+
+		setup_done(p_ca);
+		setup_done(p_control);
+		setup_done(p_lka);
+		setup_done(p_pony);
+		setup_done(p_queue);
+		setup_done(p_scheduler);
+
+		log_info("smtpd: setup done");
+
+		return smtpd();
+	}
+
+	if (!strcmp(rexec, "ca")) {
+		smtpd_process = PROC_CA;
+		setup_proc();
+
+		return ca();
+	}
+
+	else if (!strcmp(rexec, "control")) {
+		smtpd_process = PROC_CONTROL;
+		setup_proc();
+
+		/* the control socket ensures that only one smtpd instance is running */
+		control_socket = control_create_socket();
+
+		env->sc_stat = stat_backend_lookup(backend_stat);
+		if (env->sc_stat == NULL)
+			errx(1, "could not find stat backend \"%s\"", backend_stat);
+
+		return control();
+	}
+
+	else if (!strcmp(rexec, "lka")) {
+		smtpd_process = PROC_LKA;
+		setup_proc();
+
+		return lka();
+	}
+
+	else if (!strcmp(rexec, "pony")) {
+		smtpd_process = PROC_PONY;
+		setup_proc();
+
+		return pony();
+	}
+
+	else if (!strcmp(rexec, "queue")) {
+		smtpd_process = PROC_QUEUE;
+		setup_proc();
+
+		if (env->sc_queue_flags & QUEUE_COMPRESSION)
+			env->sc_comp = compress_backend_lookup("gzip");
+
+		if (!queue_init(backend_queue, 1))
+			errx(1, "could not initialize queue backend");
+
+		return queue();
+	}
+
+	else if (!strcmp(rexec, "scheduler")) {
+		smtpd_process = PROC_SCHEDULER;
+		setup_proc();
+
+		for (i = 0; i < MAX_BOUNCE_WARN; i++) {
+			if (env->sc_bounce_warn[i] == 0)
+				break;
+			log_debug("debug: bounce warning after %s",
+			    duration_to_text(env->sc_bounce_warn[i]));
+		}
+
+		return scheduler();
+	}
+
+	fatalx("bad rexec: %s", rexec);
+
+	return (1);
+}
+
+static struct mproc *
+start_child(int save_argc, char **save_argv, char *rexec)
+{
+	struct mproc *p;
+	char *argv[SMTPD_MAXARG];
+	int sp[2], argc = 0;
+	pid_t pid;
+
+	if (save_argc >= SMTPD_MAXARG - 2)
+		fatalx("too many arguments");
+
+	if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC, sp) == -1)
+		fatal("socketpair");
+
+	io_set_nonblocking(sp[0]);
+	io_set_nonblocking(sp[1]);
+
+	switch (pid = fork()) {
+	case -1:
+		fatal("%s: fork", save_argv[0]);
+	case 0:
+		break;
+	default:
+		close(sp[0]);
+		p = calloc(1, sizeof(*p));
+		if (p == NULL)
+			fatal("calloc");
+		if((p->name = strdup(rexec)) == NULL)
+			fatal("strdup");
+		mproc_init(p, sp[1]);
+		p->pid = pid;
+		p->handler = parent_imsg;
+		return p;
+	}
+
+	close(sp[1]);
+
+	if (dup2(sp[0], 3) == -1)
+		fatal("%s: dup2", rexec);
+
+	for (argc = 0; argc < save_argc; argc++)
+		argv[argc] = save_argv[argc];
+	argv[argc++] = "-x";
+	argv[argc++] = rexec;
+	argv[argc++] = NULL;
+
+	execvp(argv[0], argv);
+	fatal("%s: execvp", rexec);
+}
+
+static void
+setup_peers(struct mproc *a, struct mproc *b)
+{
+	int sp[2];
+
+	if (socketpair(AF_UNIX, SOCK_STREAM, PF_UNSPEC, sp) == -1)
+		fatal("socketpair");
+
+	io_set_nonblocking(sp[0]);
+	io_set_nonblocking(sp[1]);
+
+	if (imsg_compose(&a->imsgbuf, IMSG_SETUP_PEER, b->proc, b->pid, sp[0],
+	    NULL, 0) == -1)
+		fatal("imsg_compose");
+	if (imsg_flush(&a->imsgbuf) == -1)
+		fatal("imsg_flush");
+
+	if (imsg_compose(&b->imsgbuf, IMSG_SETUP_PEER, a->proc, a->pid, sp[1],
+	    NULL, 0) == -1)
+		fatal("imsg_compose");
+	if (imsg_flush(&b->imsgbuf) == -1)
+		fatal("imsg_flush");
+}
+
+static void
+setup_done(struct mproc *p)
+{
+	struct imsg imsg;
+
+	if (imsg_compose(&p->imsgbuf, IMSG_SETUP_DONE, 0, 0, -1, NULL, 0) == -1)
+		fatal("imsg_compose");
+	if (imsg_flush(&p->imsgbuf) == -1)
+		fatal("imsg_flush");
+
+	if (imsg_wait(&p->imsgbuf, &imsg, 10000) == -1)
+		fatal("imsg_wait");
+
+	if (imsg.hdr.type != IMSG_SETUP_DONE)
+		fatalx("expect IMSG_SETUP_DONE");
+
+	log_info("setup_done: %s[%d] done", p->name, p->pid);
+
+	imsg_free(&imsg);
+}
+
+static void
+setup_proc(void)
+{
+	struct imsgbuf *ibuf;
+	struct imsg imsg;
+        int setup = 1;
+
+	p_parent = calloc(1, sizeof(*p_parent));
+	if (p_parent == NULL)
+		fatal("calloc");
+	if((p_parent->name = strdup("parent")) == NULL)
+		fatal("strdup");
+	p_parent->proc = PROC_PARENT;
+	p_parent->handler = imsg_dispatch;
+	mproc_init(p_parent, 3);
+
+	ibuf = &p_parent->imsgbuf;
+
+	while (setup) {
+		if (imsg_wait(ibuf, &imsg, 10000) == -1)
+			fatal("imsg_wait");
+
+		switch (imsg.hdr.type) {
+		case IMSG_SETUP_KEY:
+			env->sc_queue_key = strdup(imsg.data);
+			break;
+		case IMSG_SETUP_PEER:
+			setup_peer(imsg.hdr.peerid, imsg.hdr.pid, imsg.fd);
+			break;
+		case IMSG_SETUP_DONE:
+			setup = 0;
+			break;
+		default:
+			fatal("bad imsg %d", imsg.hdr.type);
+		}
+		imsg_free(&imsg);
+	}
+
+	if (imsg_compose(ibuf, IMSG_SETUP_DONE, 0, 0, -1, NULL, 0) == -1)
+		fatal("imsg_compose");
+
+	if (imsg_flush(ibuf) == -1)
+		fatal("imsg_flush");
+
+	log_info("setup_proc: %s done", proc_title(smtpd_process));
+}
+
+static struct mproc *
+setup_peer(enum smtp_proc_type proc, pid_t pid, int sock)
+{
+	struct mproc *p, **pp;
+
+	log_info("setup_peer: %s -> %s[%u] fd=%d", proc_title(smtpd_process),
+	    proc_title(proc), pid, sock);
+
+	if (sock == -1)
+		fatalx("peer socket not received");
+
+	switch (proc) {
+	case PROC_LKA:
+		pp = &p_lka;
+		break;
+	case PROC_QUEUE:
+		pp = &p_queue;
+		break;
+	case PROC_CONTROL:
+		pp = &p_control;
+		break;
+	case PROC_SCHEDULER:
+		pp = &p_scheduler;
+		break;
+	case PROC_PONY:
+		pp = &p_pony;
+		break;
+	case PROC_CA:
+		pp = &p_ca;
+		break;
+	default:
+		fatalx("unknown peer");
+	}
+
+	if (*pp)
+		fatalx("peer already set");
+
+	p = calloc(1, sizeof(*p));
+	if (p == NULL)
+		fatal("calloc");
+	if((p->name = strdup(proc_title(proc))) == NULL)
+		fatal("strdup");
+	mproc_init(p, sock);
+	p->pid = pid;
+	p->proc = proc;
+	p->handler = imsg_dispatch;
+
+	*pp = p;
+
+	return p;
+}
+
+static int
+imsg_wait(struct imsgbuf *ibuf, struct imsg *imsg, int timeout)
+{
+	struct pollfd pfd[1];
+	ssize_t n;
+
+	pfd[0].fd = ibuf->fd;
+	pfd[0].events = POLLIN;
+	
+	while (1) {
+		if ((n = imsg_get(ibuf, imsg)) == -1)
+			return -1;
+		if (n)
+			return 1;
+
+		n = poll(pfd, 1, timeout);
+		if (n == -1)
+			return -1;
+		if (n == 0) {
+			errno = ETIMEDOUT;
+			return -1;
+		}
+
+		if (((n = imsg_read(ibuf)) == -1 && errno != EAGAIN) || n == 0)
+			return -1;
+	}
+}
+
+int
+smtpd(void) {
+	struct event	 ev_sigint;
+	struct event	 ev_sigterm;
+	struct event	 ev_sigchld;
+	struct event	 ev_sighup;
+	struct timeval	 tv;
 
 	imsg_callback = parent_imsg;
+
+	tree_init(&children);
+
+	child_add(p_queue->pid, CHILD_DAEMON, proc_title(PROC_QUEUE));
+	child_add(p_control->pid, CHILD_DAEMON, proc_title(PROC_CONTROL));
+	child_add(p_lka->pid, CHILD_DAEMON, proc_title(PROC_LKA));
+	child_add(p_scheduler->pid, CHILD_DAEMON, proc_title(PROC_SCHEDULER));
+	child_add(p_pony->pid, CHILD_DAEMON, proc_title(PROC_PONY));
+	child_add(p_ca->pid, CHILD_DAEMON, proc_title(PROC_CA));
+
 	event_init();
 
 	signal_set(&ev_sigint, SIGINT, parent_sig_handler, NULL);
@@ -785,10 +1143,6 @@ load_pki_tree(void)
 
 		if (!ssl_load_certificate(pki, pki->pki_cert_file))
 			fatalx("load_pki_tree: failed to load certificate file");
-
-		if (pki->pki_dhparams_file)
-			if (!ssl_load_dhparams(pki, pki->pki_dhparams_file))
-				fatalx("load_pki_tree: failed to load dhparams file");
 	}
 
 	log_debug("debug: init ca-tree");
@@ -817,37 +1171,6 @@ load_pki_keys(void)
 	}
 }
 
-static void
-fork_peers(void)
-{
-	tree_init(&children);
-
-	init_pipes();
-
-	child_add(queue(), CHILD_DAEMON, proc_title(PROC_QUEUE));
-	child_add(control(), CHILD_DAEMON, proc_title(PROC_CONTROL));
-	child_add(lka(), CHILD_DAEMON, proc_title(PROC_LKA));
-	child_add(scheduler(), CHILD_DAEMON, proc_title(PROC_SCHEDULER));
-	child_add(pony(), CHILD_DAEMON, proc_title(PROC_PONY));
-	child_add(ca(), CHILD_DAEMON, proc_title(PROC_CA));
-	post_fork(PROC_PARENT);
-}
-
-void
-post_fork(int proc)
-{
-	if (proc != PROC_QUEUE && env->sc_queue_key) {
-		explicit_bzero(env->sc_queue_key, strlen(env->sc_queue_key));
-		if (strcasecmp(env->sc_queue_key, "stdin") != 0)
-			free(env->sc_queue_key);
-	}
-
-	if (proc != PROC_CONTROL) {
-		close(control_socket);
-		control_socket = -1;
-	}
-}
-
 int
 fork_proc_backend(const char *key, const char *conf, const char *procname)
 {
@@ -892,7 +1215,7 @@ fork_proc_backend(const char *key, const char *conf, const char *procname)
 		if (procname == NULL)
 			procname = name;
 
-		execl(path, procname, arg, NULL);
+		execl(path, procname, arg, (char *)NULL);
 		err(1, "execl: %s", path);
 	}
 
@@ -1503,6 +1826,10 @@ imsg_to_str(int type)
 
 	CASE(IMSG_CTL_SMTP_SESSION);
 
+	CASE(IMSG_SETUP_KEY);
+	CASE(IMSG_SETUP_PEER);
+	CASE(IMSG_SETUP_DONE);
+
 	CASE(IMSG_CONF_START);
 	CASE(IMSG_CONF_END);
 
@@ -1680,6 +2007,7 @@ int
 parent_auth_getspnam(const char *username, const char *password)
 {
        struct spwd *pw;
+       char *ep;
 
        errno = 0;
        do {
@@ -1692,7 +2020,10 @@ parent_auth_getspnam(const char *username, const char *password)
                return LKA_PERMFAIL;
        }
 
-       if (strcmp(pw->sp_pwdp, crypt(password, pw->sp_pwdp)) == 0)
+       if ((ep = crypt(password, pw->sp_pwdp)) == NULL)
+	       return LKA_PERMFAIL;
+
+       if (strcmp(pw->sp_pwdp, ep) == 0)
                return LKA_OK;
 
        return LKA_PERMFAIL;
@@ -1703,6 +2034,7 @@ int
 parent_auth_pwd(const char *username, const char *password)
 {
        struct passwd *pw;
+       char *ep;
 
        errno = 0;
        do {
@@ -1715,7 +2047,10 @@ parent_auth_pwd(const char *username, const char *password)
                return LKA_PERMFAIL;
        }
 
-       if (strcmp(pw->pw_passwd, crypt(password, pw->pw_passwd)) == 0)
+       if ((ep = crypt(password, pw->pw_passwd)) == NULL)
+	       return LKA_PERMFAIL;
+
+       if (strcmp(pw->pw_passwd, ep) == 0)
                return LKA_OK;
 
        return LKA_PERMFAIL;
