@@ -1,4 +1,4 @@
-/*	$OpenBSD$	*/
+/*	$OpenBSD: table_static.c,v 1.15 2016/01/22 13:08:44 gilles Exp $	*/
 
 /*
  * Copyright (c) 2013 Eric Faurot <eric@openbsd.org>
@@ -44,14 +44,16 @@
 static int table_static_config(struct table *);
 static int table_static_update(struct table *);
 static void *table_static_open(struct table *);
-static int table_static_lookup(void *, struct dict *, const char *, enum table_service,
+static int table_static_lookup(void *, struct dict *, const char *,
+    enum table_service, union lookup *);
+static int table_static_fetch(void *, struct dict *, enum table_service,
     union lookup *);
-static int table_static_fetch(void *, struct dict *, enum table_service, union lookup *);
 static void  table_static_close(void *);
 static int table_static_parse(struct table *, const char *, enum table_type);
 
 struct table_backend table_backend_static = {
-	K_ALIAS|K_CREDENTIALS|K_DOMAIN|K_NETADDR|K_USERINFO|K_SOURCE|K_MAILADDR|K_ADDRNAME|K_MAILADDRMAP,
+	K_ALIAS|K_CREDENTIALS|K_DOMAIN|K_NETADDR|K_USERINFO|
+	K_SOURCE|K_MAILADDR|K_ADDRNAME|K_MAILADDRMAP,
 	table_static_config,
 	table_static_open,
 	table_static_update,
@@ -84,26 +86,21 @@ static int
 table_static_parse(struct table *t, const char *config, enum table_type type)
 {
 	FILE	*fp;
-	char	*buf, *lbuf;
-	size_t	 flen;
+	char	*buf = NULL;
+	size_t	 sz = 0;
+	ssize_t	 flen;
 	char	*keyp;
 	char	*valp;
 	size_t	 ret = 0;
 
-	fp = fopen(config, "r");
-	if (fp == NULL)
-		return 0;
+        if ((fp = fopen(config, "r")) == NULL) {
+                log_warn("warn: Table \"%s\"", config);
+                return 0;
+        }
 
-	lbuf = NULL;
-	while ((buf = fgetln(fp, &flen))) {
+	while ((flen = getline(&buf, &sz, fp)) != -1) {
 		if (buf[flen - 1] == '\n')
 			buf[flen - 1] = '\0';
-		else {
-			lbuf = xmalloc(flen + 1, "table_config_parse");
-			memcpy(lbuf, buf, flen);
-			lbuf[flen] = '\0';
-			buf = lbuf;
-		}
 
 		keyp = buf;
 		while (isspace((unsigned char)*keyp))
@@ -115,7 +112,8 @@ table_static_parse(struct table *t, const char *config, enum table_type type)
 		if (valp) {
 			while (*valp) {
 				if (!isspace((unsigned char)*valp) &&
-				    !(*valp == ':' && isspace((unsigned char)*(valp + 1))))
+				    !(*valp == ':' &&
+				    isspace((unsigned char)*(valp + 1))))
 					break;
 				++valp;
 			}
@@ -123,7 +121,6 @@ table_static_parse(struct table *t, const char *config, enum table_type type)
 				valp = NULL;
 		}
 
-		/**/
 		if (t->t_type == 0)
 			t->t_type = (valp == keyp || valp == NULL) ? T_LIST :
 			    T_HASH;
@@ -144,7 +141,7 @@ table_static_parse(struct table *t, const char *config, enum table_type type)
 
 	ret = 1;
 end:
-	free(lbuf);
+	free(buf);
 	fclose(fp);
 	return ret;
 }
@@ -192,8 +189,8 @@ table_static_close(void *hdl)
 }
 
 static int
-table_static_lookup(void *hdl, struct dict *params, const char *key, enum table_service service,
-    union lookup *lk)
+table_static_lookup(void *hdl, struct dict *params, const char *key,
+    enum table_service service, union lookup *lk)
 {
 	struct table   *m  = hdl;
 	char	       *line;
@@ -238,14 +235,15 @@ table_static_lookup(void *hdl, struct dict *params, const char *key, enum table_
 }
 
 static int
-table_static_fetch(void *hdl, struct dict *params, enum table_service service, union lookup *lk)
+table_static_fetch(void *hdl, struct dict *params,
+    enum table_service service, union lookup *lk)
 {
 	struct table   *t = hdl;
 	const char     *k;
 
-	if (! dict_iter(&t->t_dict, &t->t_iter, &k, (void **)NULL)) {
+	if (!dict_iter(&t->t_dict, &t->t_iter, &k, (void **)NULL)) {
 		t->t_iter = NULL;
-		if (! dict_iter(&t->t_dict, &t->t_iter, &k, (void **)NULL))
+		if (!dict_iter(&t->t_dict, &t->t_iter, &k, (void **)NULL))
 			return 0;
 	}
 
